** This file contains sas code for the paper Huang Hui Li (2018)          ;
** "Federal judge ideology: A new measure for ex-ante litigation risk "   ;
** that converts the raw data into the final dataset used in the analysis ;  


***Set up environment parameters******************************;

* Set sas library;
libname fci '/wrds/fci.test';

*Set SAS working directory;
data _null_;
  rc=dlgcdir("~/Ideas/2013_Litigations/fci/");
  put rc=; run;

  %let output_dir = ~/Ideas/2013_Litigations/fci.test/;

libname comp "/wrds/sasdata.old/comp";
libname crsp "/wrds/sasdata.old/crsp";

***Step 01:                     ******************************;
***State level control variables******************************;
***Election outcome, unemployment, gdp************************;


*Presidential election outcomes by state;
/*
we hand-collected the U.S. Senate election history from Wikipedia:
https://en.wikipedia.org/wiki/Category:Lists_of_United_States_Senators_by_state
*/
data PresElecSt0;
  infile "fci_data/Pres_Elec_State.csv"  delimiter=','  MISSOVER DSD lrecl=32767 firstobs=2;
  format state $16.;
  format statecode E1 E2 E3 E4 E5 E6 E7 E8 E9 E10 E11 E12 E13 E14 E15 E16 E17 E18 E19 
    E20 E21 E22 E23 E24 E25 E26 E27 E28 E29 E30 E31 E32 E33 E34 E35 E36 E37 E38 E39 
    E40 E41 E42 E43 E44 E45 E46 E47 E48 E49 E1984 E1988 E1992 E1996 E2000 E2004 E2008 E2012 $6.; 
  input state $ statecode $ E1 $ E2 $ E3 $ E4 $ E5 $ E6 $ E7 $ E8 $ E9 $ E10 $  E11 $ E12 $ E13 $ E14 $ E15 $ 
  E16 $ E17 $ E18 $ E19 $ E20 $ E21 $ E22 $ E23 $ E24 $ E25 $ E26 $ E27 $ E28 $ E29 $ E30 $  
  E31 $ E32 $ E33 $ E34 $ E35 $ E36 $ E37 $ E38 $ E39 $ E40 $ E41 $ E42 $ E43 $ E44 $ E45 $ 
  E46 $ E47 $ E48 $ E49 $ E1984 $  E1988 $ E1992 $ E1996 $ E2000 $ E2004 $ E2008 $ E2012 $;
  format D1988 D1992 D1996 D2000 D2004 D2008 D2012 4.;
  D1988=(E1988='D');   D1992=(E1992='D');  D1996=(E1996='D');  D2000=(E2000='D');  
  D2004=(E2004='D');  D2008=(E2008='D');  D2012=(E2012='D');  
run;


*Keep records only since 1988 GHWBush;
data PresElecst1; set PresElecSt0(keep=state statecode D1988 D1992 D1996 D2000 D2004 D2008 D2012); run;

proc sql; create table monthly0 as  select distinct datadate from comp.fundq order by datadate;


proc sql; create table PresElec2 as select distinct m.datadate, e.statecode, 
    e.D1988, e.D1992, e.D1996, e.D2000, e.D2004, e.D2008, e.D2012, case
    when intck("day", "15Nov1992"d, m.datadate) GT 0 and intck("day", "01Nov1996"d, m.datadate) LT 0 then D1992
    when intck("day", "15Nov1996"d, m.datadate) GT 0 and intck("day", "01Nov2000"d, m.datadate) LT 0 then D1996
    when intck("day", "15Nov2000"d, m.datadate) GT 0 and intck("day", "01Nov2004"d, m.datadate) LT 0 then D2000
    when intck("day", "15Nov2004"d, m.datadate) GT 0 and intck("day", "01Nov2008"d, m.datadate) LT 0 then D2004
    when intck("day", "15Nov2008"d, m.datadate) GT 0 and intck("day", "01Nov2012"d, m.datadate) LT 0 then D2008
    when intck("day", "15Nov2012"d, m.datadate) GT 0 and intck("day", "01Nov2016"d, m.datadate) LT 0 then D2012
    else D1988 end as peld
  from monthly0 m, preselecst1 e ;

proc sort data=PresElec2 nodupkeys; by datadate statecode peld; run;

data fci.preselec2; set preselec2; run;

* Unemployment rate by states;
/*
We obtained from the Local Area Unemployment Statistics (LAUS) 
by Bureau of Labor Statistics the State-level unemployment rate history 
and calculated state-level variable UNEMP. 
The data is available at https://www.bls.gov/lau/ as of November 2018.
*/
data employ0; 
  infile "fci_data/state_gdp/employ_all.txt" firstobs=4; 
  format code1  _labor _employed _unemployed $16.;  format county_state $56.;
  format fps_state fps_county stateabbr $4.;   format year 4.;
  format labor employed unemployed comma10.;  format rate 8.2;
  input code1 $ 1-15 fps_state $ 18-21 fps_county $ 25-28 county_state $ 32-78 year  82-86   
    _labor $ 92-99 _employed $ 103-112 _unemployed $ 116-123 rate 128-132;
  labor=input(_labor,comma10.);    employed=input(_employed,comma10.);  unemployed=input(_unemployed,comma10.);
  county_state=strip(county_state);
  stateabbr=scan(county_state,-1);
  if find(stateabbr,"Colu","i" ) GT 0 then stateabbr="DC";
  drop code1 _labor _employed _unemployed rate;
run;

proc sql; create table employ1 as select distinct year, fps_state, stateabbr, 
  sum(unemployed)/sum(labor) as rate_unemp
  from employ0 group by year, fps_state, stateabbr;

* GDP growth by states;
/*
we collected state-level GDP growth data from the Bureau of Economic Analysis 
which is available at http://www.bea.gov/regional to calculate the variable GDPGR.
*/  
  
*1997--2014;
%macro pyears(y1=,y2=,indcode=n);
  %do k=&y1 % to &y2;   p&indcode.&k  %end; %mend;

data gdp98_0; 
  infile "fci_data/state_gdp/allst1_9714.csv" 
    firstobs=2  delimiter="," MISSOVER DSD lrecl=32767;
  format fips $5.; format state $24.; format   %pyears(y1=1997,y2=2014) 8.;
  input fips $ state $ %pyears(y1=1997,y2=2014);  
  fps_state=substr(fips,1,2);  run;

*1963--1997;
data gdp97_0; 
  infile "fci_data/state_gdp/allst1_6397_sic2.csv" 
    firstobs=2  delimiter="," MISSOVER DSD lrecl=32767;
  format fips $5.; format state $24.; format   %pyears(y1=1963,y2=1997,indcode=s) 8.;
  input fips $ state $ %pyears(y1=1963,y2=1997,indcode=s);  
  fps_state=substr(fips,1,2);  run;

proc sort data=gdp97_0; by fps_state; run;
proc sort data=gdp98_0; by fps_state; run;
data gdp0; merge gdp97_0 gdp98_0; by fps_state; run;

%macro gdp_years;
  %do y=1964 %to 2014;
  proc sql; create table gdp_&y. as select distinct &y as year, fps_state, state, 
    %if &y LE %eval(1997) %then %do; ps&y / ps%eval(&y-1)  %end;
    %if &y GE %eval(1998) %then %do; pn&y / pn%eval(&y-1)  %end; -1 as gdpgr
  from gdp0;
  %end; 
  data gdp1; set 
   %do y=1964 %to 2014;  gdp_&y  %end;  ;
  run;
%mend;

%gdp_years;

proc sort data=gdp1;    by year fps_state; run;
proc sort data=employ1; by year fps_state; run;

*Merge into one dataset;
data state_vars(rename=(stateabbr=statecode)); merge gdp1 employ1; by year fps_state; 
  fps_state=substr(fps_state,1,2);  run;

proc sort data=state_vars; by fps_state; run;
data states_vars1; merge state_vars ; by fps_state; run;

proc sql; create table state_vars1 as select distinct s.*, m.datadate
  from states_vars1 s inner join monthly0 m on year(datadate)=s.year;

data fci.state_vars; set state_vars1; run;






***Step 02:                     ******************************;
***Federal judge data           ******************************;
***Calculate federal court ideology at circuit ***************;

/*
In May 2015, we downloaded the data of Federal judge's appointment history 
from Federal Judicial Center (FJC) in order to calculate the variable LiberalCourt. 
The data was originally located in http://fjc.gov/history/export/jb.txt. 
As of November 2018, the file location is at 
https://www.fjc.gov/sites/default/files/history/judges.csv. 
*/


data fjc_hist0; 
  infile "fci_data/history_judiciary_1789_csv.csv" delimiter=','  MISSOVER DSD lrecl=32767 firstobs=2;
  informat fjcjid 6.; informat CircuitNo Party $4.;
  informat courtName Reason $128.;
  informat courtType jdSname jdFname jdMname PresName partyName Presdt2Name party2Name _normDate ABARating 
    _recessAppDate pdfname pdlname authleg  refDate cmtAct cmtActDate HearingDate SenateVoteResult SenateVoteAN 
    SenateVoteDate  _CommissionDate  _ChiefDate3 _ChiefDate4 _RetireDate _TermDate RepNum $32.;
  format CommissionDate RecessappDate RetireDate TermDate jdate1 jdate2 yymmddn8.;
  informat ChiefDate1 ChiefDate2 4.;
input fjcjid $ jdSname $ jdFname $ jdMname $ CourtName $ CourtType $ PresName $ PartyName $   Presdt2Name $ 
  party2Name $ 
  _normDate $ ABARating $ _recessAppDate $  pdfname $ pdlname $ authleg $ refDate $ RepNum $ CmtAct $ cmtActDate $ 
  HearingDate $ SenateVoteResult $ SenateVoteAN $ SenateVoteDate $
  _CommissionDate $ chiefdate1 chiefdate2  _chiefdate3 $ _chiefdate4 $ _retiredate $ _termDate $ reason $;
  CommissionDate=input(_commissionDate,mmddyy10.);    RecessappDate=input(_recessappdate,mmddyy10.); 
  RetireDate=input(_retiredate,mmddyy10.);            Termdate=input(_termdate,mmddyy10.);
  jdate1=CommissionDate; 
  if RecessappDate ne . and (intck("day",recessappdate, commissiondate) GT 0 or commissiondate=.) 
    then jdate1=recessappdate;
  jdate2=Retiredate;     if TermDate ne . then jdate2=termdate;
  if termdate =. and length(strip(reason)) LT 2 then jdate2="31Dec2199"d; 
  Party="na";  if find(partyname,"Dem","i") GT 0 then Party="D"; if find(partyname,"Rep","i") GT 0 then Party="R";
  CircuitNo="na";                                               if find(courtname,"Federal","i")  GT 0 then CircuitNo="CF";
  if find(courtname,"First","i")    GT 0 then CircuitNo="C1";   if find(courtname,"Second","i")   GT 0 then CircuitNo="C2";
  if find(courtname,"Third","i")    GT 0 then CircuitNo="C3";   if find(courtname,"Fourth","i")   GT 0 then CircuitNo="C4";
  if find(courtname,"Fifth","i")    GT 0 then CircuitNo="C5";   if find(courtname,"Sixth","i")    GT 0 then CircuitNo="C6";
  if find(courtname,"Seventh","i")  GT 0 then CircuitNo="C7";   if find(courtname,"Eighth","i")   GT 0 then CircuitNo="C8";
  if find(courtname,"Ninth","i")    GT 0 then CircuitNo="C9";   if find(courtname,"Tenth","i")    GT 0 then CircuitNo="C10";
  if find(courtname,"Eleventh","i") GT 0 then CircuitNo="C11";  if find(courtname,"Columbia","i") GT 0 then CircuitNo="DC";
  if fjcjid NE .;
run;

proc sort data=fjc_hist0 nodupkey ; by fjcjid jdate1 jdate2 courtname; run;
data fci.fjc_hist0; set fjc_hist0; run;

*USCA court of appeals;
data fjc_hist1; set fjc_hist0; by fjcjid jdate1; 
  Partylag=lag(Party); if first.fjcjid then partylag="";
  if party="na" and partylag ne "" then party=partylag; 
  if substr(courttype,1,4)="USCA"; run; 


*Senators history;
/*
we hand-collected the U.S. Senate election history from Wikipedia:
https://en.wikipedia.org/wiki/Category:Lists_of_United_States_Senators_by_state
*/
data sen_hist0o(drop=_sdate1 _sdate2 partyname);
  infile "fci_data/senatehist_20160507.csv"  MISSOVER DSD lrecl=32767 firstobs=2;
  format state sname partyname $32.; format class $8.; format _sdate1 _sdate2 $12.; format sdate1 sdate2 yymmddn8.; 
  format Sparty $4.;
  input state $ class $ Sname $ partyname $ _sdate1  $ _sdate2 $; 
  sdate1=input(_sdate1,date9.);    sdate2=input(_sdate2,date9.); 
  state=tranwrd(state,"_"," ");
  SParty="na";  if find(partyname,"Dem","i") GT 0 then SParty="D";  if find(partyname,"Rep","i") GT 0 then SParty="R";
  if find(partyname,"Ind","i") GT 0 then SParty="I"  ; 
  if find(partyname,"Rep","i") GT 0 and  find(partyname,"Dem","i")  then SParty="na"; run;

proc sort data=sen_hist0o nodupkeys; by state sdate1 sdate2 sparty sname; run;

data states; 
  format state $24.; format statecode $6.;    infile datalines delimiter=','; 
  input state $ statecode $;
  datalines; 
Alabama,                     AL    
Alaska,                      AK    
Arizona,                     AZ    
Arkansas,                    AR    
California,                  CA    
Colorado,                    CO    
Connecticut,                 CT    
Delaware,                    DE    
District of Columbia,        DC    
Florida,                     FL    
Georgia,                     GA    
Hawaii,                      HI    
Idaho,                       ID    
Illinois,                    IL    
Indiana,                     IN    
Iowa,                        IA    
Kansas,                      KS    
Kentucky,                    KY    
Louisiana,                 LA    
Maine,                      ME    
Maryland,                   MD    
Massachusetts,              MA    
Michigan,                   MI    
Minnesota,                  MN    
Mississippi,                MS    
Missouri,                   MO    
Montana,                    MT    
Nebraska,                   NE    
Nevada,                     NV    
New Hampshire,               NH    
New Jersey,                  NJ    
New Mexico,                  NM    
New York,                    NY    
North Carolina,              NC    
North Dakota,                ND    
Ohio,                       OH    
Oklahoma,                   OK    
Oregon,                     OR    
Pennsylvania,               PA    
Rhode Island,                RI    
South Carolina,              SC    
South Dakota,                SD    
Tennessee,                  TN    
Texas,                      TX    
Utah,                       UT    
Vermont,                    VT    
Virginia,                   VA    
Washington,                 WA    
West Virginia,               WV    
Wisconsin,                  WI    
Wyoming,                    WY  
;
run;

proc sort data=states; by state; run;

data sen_hist0; merge sen_hist0o states; by state; if state="VicePresident" then statecode="XX"; 
  S_Dem=0; if SParty="D" then S_Dem=1;   S_GOP=0; if SParty="R" then S_GOP=1; run;

proc sql; create table sen_psend0 as select distinct m.datadate, h.*
  from monthly0 m inner join sen_hist0 h on m.datadate gt h.sdate1 and m.datadate lt h.sdate2;

proc sql; create table sen_psend1o as select distinct datadate,    
  sum(S_Dem)/count(*) as psend label="Pct Dem Senators", 
  sum(S_GOP)/count(*) as psenr label="Pct GOP Senators", 
  count(*) as nsen label="Num Senators", 
  sum(s_dem) as nsend label="Num Dem Senators",
  sum(s_gop) as nsenr label="Num GOP Senators"
  from sen_psend0 group by datadate order by datadate;

data sen_psend1; set sen_psend1o; 
  S_DemMajority=0; if psend GT 0.5 then S_DemMajority=1; 
  S_GOPMajority=0; if psenr GT 0.5 then S_GOPMajority=1; 
  label   S_DemMajority = "Dem GT 50"     S_GOPMajority = "GOP GT 50" ;run;

*Psend0, senate majority;
proc sql; create table fjc_hist1_psend0 as select j.*, 
  s.psend, psenr, s.S_DemMajority, s.S_GOPMajority, 
  s.nsen, s.nsend, s.nsenr, s.datadate as s_datadate
  from fjc_hist1 j left join sen_psend1 s on intck("month",j.jdate1,s.datadate) = 0
  order by j.fjcjid, j.jdate1, j.jdate2, j.courtname;

*For USCA: Keep only on-the-job Judges for each datadate 
 including the pct of senators by party as of the time of judges appointment ;
proc sql; create table fjc_pctd0 as select distinct 
  m.datadate, h.fjcjid, h.jdsname, h.jdfname,
  h.circuitno, h.party, h.jdate1, h.jdate2, h.Retiredate,
  case when m.datadate LE h.Retiredate or h.Retiredate eq . then 0 else 1 end as Senior,
  case when party="D" then 1 else 0 end as I_DEM,
  h.psend, h.psenr, h.s_demmajority, h.s_gopmajority, 
  h.nsend, h.nsenr, h.nsen
  from monthly0 m inner join fjc_hist1_psend0 h 
    on m.datadate GT h.jdate1 and m.datadate LT h.jdate2;
proc sort data=fjc_pctd0 nodupkey; by datadate circuitno fjcjid; run;

data fjc_pctd0; set fjc_pctd0;
  Sen_DI = I_DEM     * S_DemMajority;  Sen_DM = I_DEM     * (1-S_DemMajority);
  Sen_RI = (1-I_DEM) * S_GOPMajority;  Sen_RM = (1-I_DEM) * (1-S_GOPMajority);
  run;

*Create a year history of president's party;
data fjc_presParty0; set fjc_hist0; keep jdate1 party PresName; if party in ("D","R");
proc sort data=fjc_presParty0 nodupkey; by jdate1 party; run;
data fjc_presParty1; set fjc_presParty0; by jdate1 party;
  jyear=year(jdate1);
  partylag=lag(party);
  run;

data terms; 
  keep termid jyear;
  start=1784;
  do termid = 1 to 57;
    do j=1 to 4; 
     jyear=start+termid*4+j; output; 
    end;
  end;
run;

proc sort data=fjc_presparty1; by jyear jdate1; run;

data fjc_presparty2; merge fjc_presparty1 terms; by jyear; 
  keep termid PresName party jyear; if party in ("D","R","W"); run;

proc sort data=fjc_presparty2 nodupkey; by termid; run;

data fjc_presparty3; merge fjc_presparty2 terms; by termid; 
  if termid=16 then party="W"; 
  retain PresTermID;
  if termid=1 then PresTermID=0;
  PresNamelag=lag(PresName);
  if find(PresName, PresNameLag,"i") = 0 then PresTermID=PresTermID+1;
  PresParty=Party;
  keep PresTermID PresParty PresName jyear;
run;

***;
*Main data set for pctd: year-month-circuit level pctd value calculated;
proc sql; create table fjc_pctd1 as select distinct datadate, circuitno, year(datadate) as jyear,
  p.PresTermID , p.PresParty, p.PresName,
  sum(I_DEM)/count(*) as pctd, 
  sum(I_DEM) as numD, count(*) as numA,
  count(distinct fjcjid) as numWorkingJudges,
  sum(I_DEM*(1-senior))/sum(1-senior) as pctda, sum(1-senior) as numActiveJudges,
  sum(Sen_DI) as nSenDI, sum(Sen_DM) as nSenDM, sum(Sen_RI) as nSenRI, sum(Sen_RM) as nSenRM
  from fjc_pctd0 j left join fjc_presparty3 p on year(j.datadate)=p.jyear
  group by circuitno, PresTermID, datadate;
proc sort data=fjc_pctd1 nodupkey; by circuitno prestermid datadate; run;

%macro missing_replace0(vs=); 
  %let inum=1;  %do %until (%scan(&vs, &inum)=); %let v=%scan(&vs, &inum); %let inum=%eval(&inum+1);
    if &v.=. then &v=0; %end;  %mend;

data fjc_pctd1b; set fjc_pctd1; by circuitno PresTermID datadate;
  format pctd pnld 8.5; 
  if numd=1 then pnld=0; 
  if numd=2 then 
    pnld=( comb(numD,2) * comb(numA-numD,1)  ) / comb(numA,3);
  if numd GE 3 then 
    pnld=(  comb(numD,3) + comb(numD,2) * comb(numA-numD,1)  ) / comb(numA,3);
  ThreeDem1 = comb(nSenDI,3)                  / comb(numA,3) * 9;
  ThreeDem2 = comb(nSenDI,2) * comb(nSenDM,1) / comb(numA,3) * 8;
  ThreeDem3 = comb(nSenDI,1) * comb(nSenDM,2) / comb(numA,3) * 7;
  ThreeDem4 =                  comb(nSenDM,3) / comb(numA,3) * 6;
  TwoDem1   = comb(nSenDI,2)                  * comb(nSenRI,1) / comb(numA,3) *  3 ;
  TwoDem2   = comb(nSenDI,2)                  * comb(nSenRM,1) / comb(numA,3) *  4 ;
  TwoDem3   = comb(nSenDI,1) * comb(nSenDM,1) * comb(nSenRI,1) / comb(numA,3) *  2 ;
  TwoDem4   = comb(nSenDI,1) * comb(nSenDM,1) * comb(nSenRM,1) / comb(numA,3) *  3 ;
  TwoDem5   = comb(nSenDM,2)                  * comb(nSenRI,1) / comb(numA,3) *  1 ;
  TwoDem6   = comb(nSenDM,2)                  * comb(nSenRM,1) / comb(numA,3) *  2 ;
  OneDem1   = comb(nSenDI,1) * comb(nSenRI,2)                  / comb(numA,3) *(-3);
  OneDem2   = comb(nSenDM,1) * comb(nSenRI,2)                  / comb(numA,3) *(-4);
  OneDem3   = comb(nSenDI,1) * comb(nSenRI,1) * comb(nSenRM,1) / comb(numA,3) *(-2);
  OneDem4   = comb(nSenDM,1) * comb(nSenRI,1) * comb(nSenRM,1) / comb(numA,3) *(-3);
  OneDem5   = comb(nSenDI,1) * comb(nSenRM,2)                  / comb(numA,3) *(-1);
  OneDem6   = comb(nSenDM,1) * comb(nSenRM,2)                  / comb(numA,3) *(-2);
  ZeroDem1  = comb(nSenRI,3)                  / comb(numA,3) * (-9);
  ZeroDem2  = comb(nSenRI,2) * comb(nSenRM,1) / comb(numA,3) * (-8);
  ZeroDem3  = comb(nSenRI,1) * comb(nSenRM,2) / comb(numA,3) * (-7);
  ZeroDem4  =                  comb(nSenRM,3) / comb(numA,3) * (-6);
  %Missing_Replace0(vs=ThreeDem1 ThreeDem2 ThreeDem3 ThreeDem4 TwoDem1 TwoDem2 TwoDem3 TwoDem4 TwoDem5 TwoDem6
    OneDem1 OneDem2 OneDem3 OneDem4 OneDem5 OneDem6 ZeroDem1 ZeroDem2 ZeroDem3 ZeroDem4);
  PSWD=ThreeDem1+ThreeDem2+ThreeDem3+ThreeDem4+TwoDem1+TwoDem2+TwoDem3+TwoDem4+TwoDem5+TwoDem6+
    OneDem1+OneDem2+OneDem3+OneDem4+OneDem5+OneDem6+ZeroDem1+ZeroDem2+ZeroDem3+ZeroDem4;
run;

proc sort data=fjc_pctd1b out=fjc_pctd2o nodupkey;           by datadate circuitno; run;

data circuit_states;
  infile datalines; 
  format circuitno statecode $3.;
  input circuitno $ statecode $;
  datalines; 
  C1        MA  
  C1        ME  
  C1        NH  
  C1        PR  
  C1        RI  
  C10       CO  
  C10       KS  
  C10       NM  
  C10       OK  
  C10       UT  
  C10       WY  
  C11       AL  
  C11       FL  
  C11       GA  
  C2        CT  
  C2        NY  
  C2        VT  
  C3        DE  
  C3        NJ  
  C3        PA  
  C3        VI  
  C4        MD  
  C4        NC  
  C4        SC  
  C4        VA  
  C4        WV  
  C5        LA  
  C5        MS  
  C5        TX  
  C6        KY  
  C6        MI  
  C6        OH  
  C6        TN  
  C7        IL  
  C7        IN  
  C7        WI  
  C8        AR  
  C8        IA  
  C8        MN  
  C8        MO  
  C8        ND  
  C8        NE  
  C8        SD  
  C9        AK  
  C9        AZ  
  C9        CA  
  C9        HI  
  C9        ID  
  C9        MT  
  C9        NV  
  C9        OR  
  C9        WA  
  DC        DC 
; 
run;

*Expand to state;
proc sql; create table fjc_pctd2a as select distinct f.*, c.statecode
  from fjc_pctd2o(keep=circuitno datadate jyear pctd pnld pswd ) f 
  inner join circuit_states c on f.circuitno=c.circuitno;

proc sort data=fjc_pctd2a nodupkey;   by datadate circuitno statecode; run;

data fci.fjc_pctd2; set fjc_pctd2a;

proc sort data=fci.state_vars out=statevars1;by state datadate; run;


*Combine judge, presidential election, state vars;
proc sort data=fjc_pctd2a out=fjc_pctd2b;             by datadate statecode ; run;
proc sort data=fci.preselec2 out=preselec2;  by datadate statecode ; run;

data statevars1a; merge statevars1(drop=statecode) states; by state; run;


data fjc_pctd2b; set fjc_pctd2b;  statecode=strip(statecode); format statecode $3.;  length statecode $3.;    run;
data preselec2;  set preselec2;   statecode=strip(statecode); format statecode $3.;  length statecode $3.;    run;
data statevars1a;set statevars1a; statecode=strip(statecode); format statecode $3.;  length statecode $3.;    run;

proc sort data=statevars1a;  by datadate statecode ; run;

data fjc_pctd2c; merge fjc_pctd2b 
  preselec2(keep=statecode datadate peld)
  statevars1a(keep=statecode datadate fps_state state rate_unemp gdpgr rename=(rate_unemp=unemp)); 
  by datadate statecode; 
  if n(pnld,peld,gdpgr,unemp)=4;
  statecode=strip(upcase(statecode));
run;

*Merge the state-datadate file with historical state-datadate-firm-year file;

*Create a complete gvkey datadate pctd dataset;
proc sql; create table pctd_comp1 as  select gvkey, datadate,sich from comp.funda
where CONSOL='C' and INDFMT='INDL' and DATAFMT='STD' and POPSRC='D' and FIC='USA'
  and year(datadate) GE 1961; 

proc sql; create table comp0cik as 
  select gvkey, fyear, datadate, CIK, cusip
  from comp.funda where consol='C' and popsrc='D' and datafmt='STD' and indfmt='INDL';
proc sort data=comp0cik; by cik; run;

*Historical State Code;
data histState0(keep=cik datadate tdate stateadd cityhist ziphist);
  infile "fci_data/tenkidx9616.txt" delimiter='|'  MISSOVER DSD lrecl=32767 firstobs=2;
  format tcik _datadate _filingdate $10.;  format StateInc StateAdd $2.; format datadate tdate yymmddn8.; 
  format cityhist $24.;
  input tcik $ fncode $  _datadate $ _filingdate $ stateinc $ stateadd $ cityhist $ ziphist $;
  datadate=input(_datadate,yymmdd8.);
  tdate=input(_filingdate,yymmdd8.);
  cik=tcik;
  cityhist=compress(cityhist,"ABCDEFGHIJKLMNOPQRSTUVWXYZ ","lk");
  if year(tdate) GE 1994; run;
proc sort data=histstate0 nodupkey; by cik datadate; run;

data HistState1 nomatchhs; merge histstate0(in=_h) comp0cik(in=_c ); by cik ;
  if gvkey="061205" and stateadd="CA" then ziphist="90017";
  if gvkey="117881" and stateadd="MI" then ziphist="48091";
  if gvkey="144139" and stateadd="IL" then ziphist="60611";
  if find(cityhist,"san ramon","i")  GT 0 and  stateadd eq "CA" then ziphist="94583"; *not 94593;
  if find(cityhist,"basking ridge","i")  GT 0 and  stateadd eq "NJ" then ziphist="07920";
  if find(cityhist,"alpharetta","i") GT 0 and  stateadd eq "GA" then ziphist="30022"; *not 30202;
  if find(cityhist,"forest","i")     GT 0 and  stateadd eq "GA" then ziphist="30298"; *not 30050;
  if find(cityhist,"akron","i")      GT 0 and  stateadd eq "OH" then ziphist="44322"; 
  if find(cityhist,"glendale","i")   GT 0 and  stateadd eq "WI" then ziphist="53209"; 
  if find(cityhist,"kenosha","i")    GT 0  then ziphist="53142"; 
  if find(cityhist,"austin","i")     GT 0 and  ziphist eq "79746" then ziphist="78746";
  if _h and _c then output histstate1;
  if _h and not _c then output nomatchhs;  run;

proc sort data=histstate1 nodupkeys; by gvkey datadate; run;
data fci.histstate0; set histstate0; run;
data fci.histstate1; set histstate1; run;

proc sql; create table pctd_comp1_histstate as select distinct c.gvkey, c.datadate, 
  h.stateadd as statecodehist, h.cityhist, h.ziphist
  from pctd_comp1 c left join  histstate1 h on c.gvkey=h.gvkey and
    intck("day", h.datadate,c.datadate) between -750 and 360
  group by c.gvkey, c.datadate   
  having abs(intck("day", h.datadate,c.datadate))=min(abs(intck("day", h.datadate,c.datadate) )); 

*compsic current state;
proc sql;  create table pctd_compsic as  
  select distinct gvkey, sic, addzip as addzip0, loc,
   substr(addzip,1,5) as addzip, city from comp.company;

data zipState(keep=zip stabbr name city);
  infile "fci_data/zbp12totals.txt" delimiter=',' MISSOVER DSD lrecl=32767 firstobs=2;
  input zip $ name $ empflag $ emp_nf $ emp $ qp1_nf $ qp1 $ ap_nf $ ap $ est $ city $ stabbr $ city_name $;
  if length(strip(zip)) eq 5;  run;

proc sql; create table citystate as select distinct city, stabbr from zipstate;

proc sql; create table pctd_compsic1o as
  select distinct c.*, z.stabbr as stabbr1, length(strip(z.stabbr)) as len1, s.stabbr as stabbr2, z.zip as zip1,
    length(strip(s.stabbr)) as len2, s.city as city2
  from pctd_compsic c left join ZIPSTATE z on substr(c.addzip,1,5)=substr(z.zip,1,5)
    left join citystate s on strip(lowcase(c.city))=strip(lowcase(s.city));

data pctd_compsic1o; set pctd_compsic1o; _o=0;
  if length(strip(stabbr1)) GE 2 then do;            
    StateCode = stabbr1; _o=1; end;  run;
proc sort data=pctd_compsic1o; by gvkey _o; run;
data pctd_compsic1; set pctd_compsic1o; by gvkey _o; if first.gvkey then output; run;

proc sql; create table pctd_comp1ao as select distinct c.gvkey, c.datadate, 
  d.sich,  c.statecodehist, c.cityhist, c.ziphist,
  s.statecode as statecode_comp, s.sic, s.city as city_comp, s.addzip as zip_comp, s.loc as loc_comp
  from pctd_comp1_histstate c 
  left join pctd_comp1 d on c.gvkey=d.gvkey and c.datadate=d.datadate
  left join pctd_compsic1 s on c.gvkey=s.gvkey;

proc sort data=pctd_comp1ao; by gvkey datadate; run;

data pctd_comp1a; set pctd_comp1ao; by gvkey datadate;
  fps=0;
  if sich eq . then sich=input(sic,4.);  sic2=strip(substr(put(sich,z4.),1,2));  if sic2="" then sic2="00";
  sic1=strip(substr(put(sich,z4.),1,1));  if sic1="" then sic1="0";
  if sich ge 2833 and sich le 2836 then fps=1;  if sich ge 8731 and sich le 8734 then fps=1;
  if sich ge 3570 and sich le 3577 then fps=1;  if sich ge 7370 and sich le 7374 then fps=1;
  if sich ge 3600 and sich le 3674 then fps=1;  if sich ge 5200 and sich le 5961 then fps=1; 
  format statecode $2.; statecode = statecodehist;
  statecode_lag=lag(statecodehist); if first.gvkey then statecode_lag="";
  if statecode="" then statecodehist=statecode_lag; 
  if StateCode="" then StateCode=StateCode_comp;  
  format city $24.; city=cityhist;
  city_lag=lag(cityhist); if first.gvkey then city_lag="";
  if city="" or find(city,"na") then city=city_lag;
  if city="" or find(city,"na") then city=city_comp;
  city=lowcase(trim(city)); 
  city=compress(city,"abcdefghijklmnopqrstuvwxyz","ki");
  ziphistlag=lag(ziphist); if first.gvkey then ziphistlag="";
  if ziphist="" or find(ziphist,"na") GT 0 then ziphist=ziphistlag;
  if ziphist="" or find(ziphist,"na") GT 0 then ziphist=zip_comp;
  ziphnum = input(ziphist,5.);
  if substr(ziphist,1,1) eq " " or length(strip(ziphist)) EQ 0 then ziphist="EMPTY";
  if gvkey="147891" and statecode="na" then do; 
    statecode="FL"; ziphist="33576";
  end;
  LOC_nonUS=0; if not (loc_comp="USA" or loc_comp="") then loc_nonUS=1;
  if length(strip(statecode)) GT 0 and LOC_nonUS=0;  run;

proc sort data=pctd_comp1a nodupkey; by gvkey datadate; run;
data fci.pctd_comp1a; set pctd_comp1a; run;

*Export the firm-year pctd pnld dataset;
proc sql; create table compa1elr_usca as   select c.gvkey, c.datadate, c.fps, c.StateCode,
  e.datadate as datedate_elr,
  e.CircuitNo,  e.pctD, e.pnld, e.pswd, e.peld, e.unemp, e.gdpgr, e.fps_state
  from pctd_comp1a c inner join fjc_pctd2c e on
  intck("month",c.datadate, e.datadate) eq 0
  and c.statecode =e.statecode  order by c.gvkey, c.datadate;

proc sort data=compa1elr_usca nodupkey; by gvkey datadate; run;


data fci.compa1elr_usca; set compa1elr_usca; run;

***Step 03:                     ******************************;
***Federal judge data, district court ************************;
***Using zip code to match firm with district court **********;

*USDC district courts; 
data fjc_usdc1; 
  set fci.fjc_hist0;  *from fci01b_usca.sas;
  by fjcjid jdate1; 
  Partylag=lag(Party); if first.fjcjid then partylag="";
  if party="na" and partylag ne "" then party=partylag; 
  if substr(courttype,1,4)="USDC"; run;

*For USDC : district court democrats: DCD;
proc sql; create table fjc_dcd0 as select distinct m.datadate, 
  h.fjcjid, h.jdsname, h.jdfname, h.courtname,
  h.party, h.jdate1, h.jdate2, h.retiredate,
  case when m.datadate LE h.Retiredate or h.Retiredate eq . then 0 else 1 end as Senior,
  case when party="D" then 1 else 0 end as I_DEM
  from monthly0 m inner join fjc_usdc1 h 
    on m.datadate GT h.jdate1 and m.datadate LT h.jdate2;

data fjc_dcd0a; set fjc_dcd0;
  if find(courtname, "U. S. District Court,", "i") GT 0 then
    courtname=substr(courtname,23); run;

proc sort data=fjc_dcd0a nodupkeys; by datadate fjcjid courtName; run;

*All DC judges;
proc sql; create table fjc_dcd1 as select distinct datadate, courtname, year(datadate) as jyear, 
  sum(i_dem)/count(*) as dcd "Perentage of District Court Judge - Democrat", 
  sum(i_dem) as numdcd, count(*) as numdca, 
  count(distinct fjcjid) as numdcworkingjudges,
  sum(1-senior) as numdcactivejudges
  from fjc_dcd0a j /*left join fjc_presparty3 p on year(j.datadate)=p.jyear*/
  group by courtname, datadate
  order by courtname, datadate;

data fjc_DisCtDem; set fjc_dcd1;
  courtname=lowcase(courtname); 
  if find(courtname,"district of south carolina") GT 0 then 
          courtname="district of south carolina";  run;
*on October 7, 1965, by 79 Stat. 951, South Carolina was reorganized as a single judicial district court;
*eastern district of south carolina
 western district of south carolina  => only district of south carolina
 eastern district of illinois => ceased to exist in October 2, 1978
;

/*
For district courts, we retrieved the federal district court 
and zip code mapping data from the website of the Administrative Office of 
the U.S. Courts, available at http://www.uscourts.gov/court-locator.
*/

*Reading the zip-district court file;
*standardize the court name in the court-zip file;

data ZipDisCt0;
  infile "fci_data/zip_district_courts20160730.txt"  delimiter='|'  MISSOVER DSD lrecl=32767 firstobs=2;
  format zip $5.; format DistCtLong CourtNamewdistrict wdistrict $48.; *format dcn1 dcn2 dcn3 dcn4 dcn5 $12.;
  input zip $ DistCtLong $; 
  format word $char18.;  count=0;  wcourt=0; lendistct=0; wdistrict="";
  numzip=input(zip,5.);
  do until(word=' ');
    count=count+1;
    word = scan(distCtLong, count, '-'); 
    if find(word,'court') GT 0 then wcourt=count;
  end;
  word_distct = scan(distctlong, wcourt-2, '-');
  if find(word_distct,"western") + find(word_distct,"eastern") +
     find(word_distct,"northern") + find(word_distct,"southern") + 
     find(word_distct,"middle") + find(word_distct,"central") GT 0 then do; 
    wdistrict=word_distct;
    lendistct = 3; 
    courtname = strip(wdistrict) || " district of " || substr(distctlong, 1, find(distctlong, strip(wdistrict)) -2);
  end; else do; 
    wdistrict='district';
    lendistct=2; 
    courtname = "district of "|| substr(distctlong, 1, find(distctlong, strip(wdistrict)) -2); end;
  courtname = tranwrd(courtname,'-', " ");
  if find(courtname,"columbia") GT 0 then courtname="u. s. district court for the district of columbia";
run;


*First set of output;
proc sql; create table fjc_disctdem1 as select distinct f.courtname, f.datadate, f.dcd, z.zip
  from fjc_disctdem f inner join zipdisct0 z on f.courtname=z.courtname;

proc sort data=fjc_disctdem1; by courtname; run;

proc sql; create table compa1elr_usdc1 as select distinct c.gvkey, c.datadate, c.ziphist, e.zip, 
 e.courtname as DisCourt,  e.dcd
 from fci.pctd_comp1a c inner join fjc_disctdem1 e on intck("month",c.datadate, e.datadate) eq 0
  and  substr(c.ziphist,1,5)=substr(e.zip,1,5)   order by c.gvkey, c.datadate;

*Repair the non-matched zip code record;
*For unfound zips, using range to assign;
data zip_full(keep=zip numzip);
  do i = 1 to 99999; zip=put(i,z5.); 
    numzip=i;
  output; end; run;

proc sort data=zipdisct0; by courtname numzip; run;
data zipdisct0s(keep=courtname zip ziplag numzip numziplag); 
  set zipdisct0; by courtname; 
  ziplag = lag(zip); if first.courtname then ziplag=zip; 
  numzip=input(zip,5.); numziplag=input(ziplag,5.); run; 
proc sort data=zipdisct0s; by numzip; run;
data zipdisct0s1; set zipdisct0s; by numzip; 
  courtnamelag=lag(courtname);
  if courtname=courtnamelag then output;run;

proc sql; create table zipdisct0repair as select distinct c.courtname, z.zip
  from zipdisct0s1 c inner join zip_full z on z.numzip gt c.numziplag and z.numzip LE c.numzip;
proc sort data=zipdisct0repair nodupkey; by courtname zip; run;


*Second step;
proc sql; create table fjc_disctdem1repair as select distinct f.courtname, f.datadate, f.dcd,  z.zip
  from fjc_disctdem f inner join zipdisct0repair z on f.courtname=z.courtname;

proc sql; create table compa1elr_usdc2a as select distinct c.gvkey, c.datadate, c.ziphist
   from fci.pctd_comp1a c where not (c.ziphist in select distinct zip from fjc_disctdem1);

proc sql; create table compa1elr_usdc2b as select distinct c.gvkey, c.datadate, c.ziphist, e.zip,
  e.courtname as DisCourt,  e.dcd
  from compa1elr_usdc2a c inner join fjc_disctdem1repair e on intck("month",c.datadate, e.datadate) eq 0
  and  substr(c.ziphist,1,5)=substr(e.zip,1,5) order by c.gvkey, c.datadate;

*Create a complete gvkey datadate pctd dataset;
proc sql; create table pctd_comp1 as  select gvkey, datadate,sich from comp.funda
where CONSOL='C' and INDFMT='INDL' and DATAFMT='STD' and POPSRC='D' and FIC='USA'
  and year(datadate) GE 1961; 

proc sql; create table pctd_comp1_histstate as select distinct c.gvkey, c.datadate, 
  h.stateadd as statecodehist, h.cityhist, h.ziphist
  from pctd_comp1 c left join  fci.histstate1 h on c.gvkey=h.gvkey and  intck("day", h.datadate,c.datadate) 
    between -750 and 360
  group by c.gvkey, c.datadate   
  having abs(intck("day", h.datadate,c.datadate))=min(abs(intck("day", h.datadate,c.datadate) )); 


*Third step;
proc sql; create table compa1elr_usdc3a as select distinct c.gvkey, c.datadate, c.ziphist, c.city
   from fci.pctd_comp1a c where not (c.ziphist in (select distinct f.zip from fjc_disctdem1 f
   union select distinct c.zip from compa1elr_usdc2b c));

proc sql; create table ziph as 
  select distinct ziphist as zip, cityhist 
  from fci.histstate0    order by zip; 
proc sql; create table zipj as 
   select distinct zip, courtname  
   from fjc_disctdem1 order by zip; 
data zipmerge; merge ziph(in=_h) zipj(in=_j); by zip; if _h and _j; run; 
proc sort data=zipmerge out=city_courtname(keep=cityhist courtname) nodupkeys; 
  by cityhist courtname; run;

proc sort data=city_courtname out=city_courtname1 nodupkeys; by courtname cityhist; run;

data fjc_disctdem2;  merge fjc_disctdem1 city_courtname1; by courtname;
  cityhist=lowcase(trim(cityhist)); 
  cityhist=compress(cityhist,"abcdefghijklmnopqrstuvwxyz","ki");
run; 

proc sql; create table compa1elr_usdc3b as select distinct c.gvkey, c.datadate, 
  e.courtname as DisCourt,   e.dcd
  from compa1elr_usdc3a c inner join fjc_disctdem2 e on intck("month",c.datadate, e.datadate) eq 0
  and  substr(c.city,1,4)=substr(e.cityhist,1,4) order by c.gvkey, c.datadate;


*Fourth step;
proc sql; create table compa1elr_usdc4a as select distinct gvkey from compa1elr_usdc3a where
  not gvkey in (select distinct gvkey from compa1elr_usdc3b);

data compa1elr_usdc4a1; 
  merge compa1elr_usdc4a(in=_a) fci.pctd_comp1a(keep=gvkey datadate ziphist city statecode);
  by gvkey;   format courtname $128.;
  datadate = intnx("year",datadate,-1,"sameday"); 
  if _a then do;
    if statecode="GA" and find(ziphist,"312")=1 then do; courtname="middle district of georgia"; output; end;
    if statecode="GA" and find(ziphist,"300")=1 then do; courtname="northern district of georgia"; output; end;
    if statecode="GA" and find(ziphist,"301")=1 then do; courtname="northern district of georgia"; output; end;
    if statecode="GA" and find(ziphist,"302")=1 then do; courtname="northern district of georgia"; output; end;
    if statecode="GA" and find(ziphist,"303")=1 then do; courtname="northern district of georgia"; output; end;    
    if statecode="MA" then do; courtname="district of massachusetts"; output; end;
    if statecode="AR" then do; courtname="eastern district of arkansas"; output; end;
    if statecode="AR" then do; courtname="western district of arkansas"; output; end;
    if statecode="AK" then do; courtname="district of alaska"; output; end;
    if statecode="CA" and find(ziphist,"945")=1 then do; courtname="northern district of california"; output; end;
    if statecode="CA" and find(ziphist,"1")=1 then do; courtname="northern district of california"; output; end;
    if statecode="CA" and find(ziphist,"2")=1 then do; courtname="northern district of california"; output; end;
    if statecode="CA" and find(city,"beverly","i")=1 then do; courtname="central district of california"; output; end;
    if statecode="FL" and find(ziphist,"320")=1 then do; courtname="middle district of florida"; output; end;
    if statecode="FL" and find(ziphist,"331")=1 then do; courtname="southern district of florida"; output; end;
    if statecode="FL" and find(ziphist,"339")=1 then do; courtname="middle district of florida"; output; end;
    if statecode="IA" and find(ziphist,"500")=1 then do; courtname="southern district of iowa"; output; end;
    if statecode="IL" and find(ziphist,"600")=1 then do; courtname="northern district of illinois"; output; end;
    if statecode="IL" and find(ziphist,"606")=1 then do; courtname="northern district of illinois"; output; end;
    if statecode="IL" and find(ziphist,"999")=1 then do; courtname="northern district of illinois"; output; end;
    if statecode="IL" and find(ziphist,"0")=1 then do; courtname="northern district of illinois"; output; end;
    if statecode="KS" then do; courtname="district of kansas"; output; end;
    if statecode="OK" and find(ziphist,"731")=1 then do; courtname="western district of oklahoma"; output; end;
    if statecode="TX" and find(ziphist,"787")=1 then do; courtname="western district of texas"; output; end;
    if statecode="WI" and find(ziphist,"534")=1 then do; courtname="eastern district of wisconsin"; output; end;
    if statecode="WI" and find(city,"glendale")=1 then do; courtname="eastern district of wisconsin"; output; end;
    if statecode="NJ" then do; courtname="district of new jersey"; output; end;
    if find(city,"crawley") GT 0 then do; courtname="southern district of west virginia"; output; end;
  end;
  run;

proc sql; create table compa1elr_usdc4b as select distinct c.gvkey, c.datadate, c.courtname as DisCourt, e.dcd 
  from compa1elr_usdc4a1 c inner join fjc_disctdem e on  abs(intck("day",c.datadate, e.datadate)) LE 10
    and find(e.courtname,trim(c.courtname)) GT 0 order by c.gvkey, c.datadate;

*Combine steps one two three four; 
data compa1elr_usdc; 
  set  compa1elr_usdc1  (where=(dcd ne .))  
       compa1elr_usdc2b (where=(dcd ne .))
       compa1elr_usdc3b (where=(dcd ne .))
       compa1elr_usdc4b (where=(dcd ne .));  run;

proc sort data=compa1elr_usdc nodupkey; by gvkey datadate; run;

data fci.compa1elr_usdc; set compa1elr_usdc; run;

***Step 04:                     ******************************;
***SCAC lawsuit data and lawsuit level controls **************;
***Matching case with firm by name             ***************;
/*
To obtain shareholder class action lawsuit filing and outcome data 
(e.g., case title, class period dates, case outcome, settlement amount, 
defendant company name, sector, industry, ticker, market status), 
we collected the data in May 2015 from lawsuit case webpages hosted 
by the Securities Class Action Clearinghouse (SCAC) website, 
which is available at http://securities.stanford.edu.
*/


data cases0; 
  infile "fci_data/scac2014_cases20150611.csv" delimiter='09'x MISSOVER DSD lrecl=32767 firstobs=2;
  informat NID $7.; informat Defendant $246.; informat CaseStatus $11.;  informat DateReview $10.; 
  informat CaseTitle $384.;  informat Sector $23.;  informat Industry $35.; informat Headquarter $26.; 
  informat Ticker $12.; informat Exchange $24.; informat MarketStatus $19.;   
  informat Court_FIC $22.;  informat Judge_FIC $41.;   informat DateFile_FIC $12.;
  informat CPS_FIC $12.;  informat CPE_FIC $12.;  informat Plaintiff1_FIC $72.;  informat ComplaintURL_FIC $1024.;
  informat Court_REF _settleamt $22.;  informat Judge_REF $41.;   informat DateFile_REF $12.;
  informat CPS_REF $12.;  informat CPE_REF $12.;  informat Plaintiff1_REF $72.;  informat ComplaintURL_REF $1024.;
  informat ComplaintURL_Other $1024.; 
  input NID $ Defendant $ CaseStatus $ _SettleAmt $ DateReview $ CaseTitle $ Sector $ Industry $ 
    Headquarter $ Ticker $ Exchange $ MarketStatus $ Court_FIC $ Judge_FIC $ DateFile_FIC $ CPS_FIC $ 
    CPE_FIC $ Plaintiff1_FIC $ ComplaintURL_FIC $ Court_REF $ Judge_REF $ DateFile_REF $ CPS_REF $ CPE_REF $ 
    Plaintiff1_REF $ ComplaintURL_REF $ ComplaintURL_Other; 
  cyear=year(input(datefile_fic,mmddyy10.));
  T4_private=0;
  if find(marketstatus,"private","i") GE 1 then t4_private=1;
  if find(exchange,"NASDAQ","i") + find(exchange, "New York SE","i")+find(exchange,"American SE","i") GT 0 
    then t4_private=0;
  if find(exchange,"NASDAQ","i") + find(exchange, "New York SE","i")+find(exchange,"American SE","i") EQ 0 
    then t4_private=1;
  FirmNotDeft=0;   if find(casetitle,scan(defendant,1),"i") GE 2 then FirmNotDeft=1; 
  TitlerLonger=0; if length(casetitle) - length(defendant) GE 30 and find(casetitle,':') GT 1 then titlelonger=1;
  FundsCase =0; if find(casetitle,"fund","i") GE 1 then FundsCase=1;
  T4_nonus=0; if  find(headquarter,"United States")=0 then t4_nonus=1;
  if court_ref="-99" then court_ref=court_fic;
run;

%macro inputDate(var);
 if length( _&var) GT 5 then &var=input(_&var,mmddyy10.);
%mend inputDate;

data cases; set cases0(rename=(CPE_FIC=_CPE_FIC CPE_REF=_CPE_REF CPS_FIC=_CPS_FIC CPS_REF=_CPS_REF 
    DateFile_FIC=_DateFile_FIC DateFile_REF=_DateFile_REF DateReview=_DateReview));
  _l = length(Defendant);
  if length(Defendant) GE 8 then do;
    if find(substr(Defendant,1,4),"THE ","i")=1 then Defendant = substr(Defendant,5,length(Defendant)-4); 
  end;   if length(Defendant) GE 7 then do;
    if find(substr(Defendant,length(Defendant)-5,6),", Inc.","i")>0 then do; 
        Defendant = substr(Defendant,1,length(Defendant)-6); end;
      else do; 
    if find(substr(Defendant,length(Defendant)-5,6),", Ltd.","i")>0 then 
      Defendant = substr(Defendant,1,length(Defendant)-6); end;
  end;
  if length(Defendant) GE 6 then do;
    if find(substr(Defendant,length(Defendant)-4,5)," Inc." ,"i")>0 then 
      Defendant = substr(Defendant,1,length(Defendant)-5);
  end;
  if length(Defendant) GE 14 then do;
    if find(substr(Defendant,length(Defendant)-11,12)," Corporation","i")>0 then do;
      *put Defendant; Defendant = substr(Defendant,1,length(Defendant)-11) || 'Corp'; *put Defendant; end;
  end;
  format CPE_FIC CPE_REF CPS_FIC CPS_REF DateFile_FIC DateFile_REF DateReview mmddyy10.;
  %inputDate(var=CPE_FIC); %inputDate(var=CPE_REF); %inputDate(var=CPS_FIC); %inputDate(var=CPS_REF);
  %inputDate(var=DateFile_FIC); %inputDate(var=DateFile_REF); %inputDate(var=DateReview);
  * Repair missing date from FIC => REF, and FIC <= REF;
  if CPS_REF = . and CPS_FIC ne . then CPS_REF=CPS_FIC;  if CPE_REF = . and CPE_FIC ne . then CPE_REF=CPE_FIC;
  if CPS_FIC = . and CPS_REF ne . then CPS_FIC=CPS_REF;  if CPE_FIC = . and CPE_REF ne . then CPE_FIC=CPE_REF;
  if DateFile_FIC=. and DateFile_REF ne . then DateFile_FIC = DateFile_REF;
  if DateFile_REF=. and DateFile_FIC ne . then DateFile_REF = DateFile_FIC;
  drop _CPS_FIC _CPE_FIC _CPS_REF _CPE_REF _DateFile_FIC _DateFile_REF;
run;

*If a case involves Rule 10b-5 violation;
data tenbfive0;
  infile "fci_data/scac2014_20150611_check10b5ipo20170225.csv"
  delimiter='09'x MISSOVER DSD lrecl=32767 firstobs=2;
  format NID $7.; format is10b5 isipo isipoalloc is14a inflate is8k isgaapv isinsid isBond isAnalyst isAReport 
     isFund isMFund isHFund isTrust ExcgAct MisstateOmit Is21MC92 ispdf 8.;
  input NID $ is10b5 isipo isipoalloc is14a inflate is8k isgaapv isinsid isBond isAnalyst isAReport isFund 
    isMFund isHFund isTrust ExcgAct MisstateOmit Is21MC92 ispdf;
run;


*Settlement amount manually collected from the SCAC website http://http://securities.stanford.edu and manually verified by RAs;
data settle0(keep=nid settlem); 
  infile "fci_data/Settlements.csv" 
  delimiter=","  MISSOVER DSD lrecl=32767 firstobs=2;
  format nid $7.;    format settleamtm 8.3;
  input NID $ _settleamtm $; 
  SettleM=input(strip(_settleamtm),best32.); 
  run;
proc sort data=settle0 nodupkeys; by nid; run;

*Dismissal and Appeal collected from the SCAC website http://http://securities.stanford.edu ;
data appeal0(keep=nid dis disp apl appeal aplold);
  infile "fci_data/dismiss_appeal20170212.csv"
  delimiter=","  MISSOVER DSD lrecl=32767 firstobs=2;
  format nid $7.;    format dis disp apl 8.3;
  input NID $ dis disp apl aplold;  
  appeal=0; if apl GT 0 then appeal=1; 
  if aplold=2 then aplold=1;
run;
proc sort data=appeal0 nodupkeys; by nid; run;

proc sort data=cases;     by nid; run;
proc sort data=tenbfive0; by nid; run;

data cases0b; merge cases(in=_c) tenbfive0 settle0 appeal0; by nid;  
  t4_nonuslisted=0; if t4_nonus+t4_private GT 0 then t4_nonuslisted=1;
  KS_IFA=0; if sum(isipoalloc,isAReport,isMFund, isHFund,isTrust) * titlelonger GT 0  then KS_IFA=1;
  ifa=0;    if sum(isipoalloc,isAReport,isMFund, isHFund) GT 0 then ifa=1;
  KS_10b5=0;if is10b5 GT 0 or (MisstateOmit GT 0 and is14a=0) then KS_10b5=1;
  if settlem GT 0 then do; disp=0; dis=0; end;  format CASE_PATH $50.;
  Case_PATH="NA";
  if appeal =. then appeal=0;
  if casestatus eq "DISMISSED" then do; 
    if appeal EQ 0 then CASE_PATH = "Dismissed at District & Closed";
    if appeal EQ 1 then CASE_PATH = "Dismissed at District & Appealed & ... & Closed";    end;
  if casestatus eq "SETTLED" then do;
    if appeal EQ 0 then CASE_PATH = "Settled at District";
    if appeal EQ 1 then CASE_PATH = "Dismissed at District & Appealed & ... & Settled";   end;
  if casestatus eq "ONGOING" then do;
    if appeal EQ 0 then CASE_PATH = "Ongoing at District";
    if appeal EQ 1 then CASE_PATH = "Dismissed at District & Appealed & Ongoing";          end;
  if year(datefile_fic) LE 2014 and year(datefile_fic) GE 1996; 
run;

data fci.cases0b; set cases0b; run;

*Match cases with crsp.stocknames;
data stocknames; set crsp.stocknames;
  format exchcdn $char12.;
  if exchcd = 1 then exchcdn="New York SE";  * if exchcd = 1 then exchcdn="New York SE";
  if exchcd = 2 then exchcdn="American SE";
  if exchcd = 3 then exchcdn="NASDAQ";
  if exchcd GE -2 and exchcd LE 0 then exchcdn="unusual";
  if exchcd = 20 then exchcdn="OTC-BB";
  if ticker ne "";
run;

%put Collect all probable matches;
proc sql; create table cases_ccm0 as
  select distinct c.*, h.ticker as ticker_crsp, h.exchcd, h.exchcdn, h.comnam,h.permno,h.st_date,
    complev(c.ticker, h.ticker,"i")/ (length(c.ticker) + length(h.ticker)) *2 as tickerdiff
  from cases0b c 
    left join stocknames h
    on find(compress(h.comnam), scan(c.Defendant,1),"it",1)>0 and strip(h.exchcdn)=strip(c.exchange)
    and DateFile_FIC GE year(h.st_date)  
  order by c.NID, c.ticker, tickerdiff  ;

data cases_ccm0a; set cases_ccm0; 
  by NID ticker; retain tickerdiffidx;
    if first.NID then tickerdiffidx=0;   tickerdiffidx= tickerdiffidx+1;
    if tickerdiff = 0 then tickerdiffidx=5;
    if tickerdiffidx LE 5 then output; run;

%put Compuste a summary _dist variable ;
data cases_ccm1; set cases_ccm0a;
   format word $char100.;
   _dist=1;    count=0; _tic=0;
   do until(word=' ');
      count=count+1;
      word = scan(Defendant, count);
      if find(comnam, word||" ", "it") =0 then _dist=_dist+8*(0.5**count);   end;
   if find(defendant, comnam,"it",1)=1 then   _dist=_dist /2 ; 
   _dist = _dist + complev(Defendant, comnam,"i") *2 / (length(Defendant)+length(comnam));
   if tickerdiff=0 then _dist=_dist/2;
   _dist = _dist *(1+tickerdiff);   
   if tickerdiff=0 then _dist=_dist/10/2;
   if tickerdiff>0 and substr(ticker,1,1) = substr(ticker_crsp,1,1) then do; _tic=1; _dist=_dist/2; end;
   if tickerdiff>0 and substr(ticker,1,2) = substr(ticker_crsp,1,2) then do; _tic=2; _dist=_dist/2; end;
   if Exchange NE "NASDAQ" and Exchange NE "American SE" and Exchange NE "New York SE" then _dist = _dist + 10;
   drop count;  run;


*Manual adjustments, where we manually match cases to firm's permno;
%macro sp(_n,_p);    if NID=&_n and PERMNO=&_p then do;  _dist=_dist /10 ;   end;  %mend sp;

data cases_ccm1a; set cases_ccm1; manual=0;
  %sp(_n="103387",_p=51617);  %sp(_n="105167",_p=10833);  %sp(_n="104388",_p=21566);  %sp(_n="104607",_p=11745);
  %sp(_n="102705",_p=89027);  %sp(_n="102626",_p=69032);  %sp(_n="103871",_p=85226);  %sp(_n="102903",_p=86462);
  %sp(_n="103175",_p=12177);  %sp(_n="104200",_p=81120);  %sp(_n="104684",_p=86520);  %sp(_n="101717",_p=75318);
  %sp(_n="102527",_p=86326);  %sp(_n="102925",_p=81910);  %sp(_n="105167",_p=18033);  %sp(_n="102638",_p=11122);  
  %sp(_n="100125",_p=80197);  %sp(_n="100759",_p=65445);  %sp(_n="104423",_p=89350);  %sp(_n="104554",_p=90352);     
  %sp(_n="102792",_p=10822);  %sp(_n="105063",_p=32636);  %sp(_n="104984",_p=68591);
  %sp(_n="102504",_p=11042);  %sp(_n="104309",_p=68144);  %sp(_n="102516",_p=49614);  %sp(_n="103589",_p=77769);  
  %sp(_n="100795",_p=80331);  %sp(_n="103605",_p=77979);  %sp(_n="104330",_p=91153);  %sp(_n="104306",_p=91576);  
  %sp(_n="104268",_p=69761);  %sp(_n="104258",_p=69761);  %sp(_n="105034",_p=24505);  %sp(_n="100739",_p=77603);  
  %sp(_n="101509",_p=43385);  %sp(_n="102982",_p=79209);  %sp(_n="101391",_p=76512);  %sp(_n="102921",_p=87585);
  %sp(_n="101181",_p=17807);  %sp(_n="102776",_p=70519);  %sp(_n="101093",_p=85673);  %sp(_n="104249",_p=89199);  
  %sp(_n="101215",_p=82218);  %sp(_n="102889",_p=75278);  %sp(_n="104071",_p=70519);  %sp(_n="101459",_p=87770);
  %sp(_n="102820",_p=11554);  %sp(_n="101283",_p=81491);  %sp(_n="104901",_p=86867);  
  %sp(_n="104143",_p=92715);  %sp(_n="102798",_p=71175);  %sp(_n="102153",_p=86425);  %sp(_n="103901",_p=12503);
  %sp(_n="103343",_p=44150);  %sp(_n="104230",_p=75180);  %sp(_n="101339",_p=71984);  %sp(_n="102543",_p=69032);
  %sp(_n="105179",_p=11500);  %sp(_n="100951",_p=24360);  %sp(_n="102838",_p=14322);  %sp(_n="104431",_p=79351);
  %sp(_n="104840",_p=12188);  %sp(_n="103822",_p=76038);  %sp(_n="103386",_p=77464);  %sp(_n="101371",_p=75757); 
  %sp(_n="105150",_p=89475);  %sp(_n="102368",_p=85160);  %sp(_n="102308",_p=86862);  %sp(_n="102309",_p=78032);
  %sp(_n="103094",_p=79528);  %sp(_n="101689",_p=86642);  %sp(_n="100020",_p=81099);  %sp(_n="104264",_p=89258);
  %sp(_n="104976",_p=27828);  %sp(_n="101874",_p=86544);  
  %sp(_n="101078",_p=83861);  %sp(_n="104393",_p=79421);  %sp(_n="102451",_p=89930);  %sp(_n="101639",_p=89736);
  %sp(_n="102100",_p=79532);  %sp(_n="101534",_p=34535);  %sp(_n="101046",_p=82168); 
  %sp(_n="101104",_p=90420);  %sp(_n="104960",_p=90601);  %sp(_n="104773",_p=86721);
  %sp(_n="102957",_p=81195);  %sp(_n="102958",_p=11483);
  %sp(_n="103764",_p=16396);  %sp(_n="102890",_p=88155);  %sp(_n="101007",_p=80391);  %sp(_n="104432",_p=50200);  
  %sp(_n="101581",_p=77483);  %sp(_n="105100",_p=88906);  %sp(_n="104292",_p=91512);  %sp(_n="104288",_p=25487); 
  %sp(_n="104206",_p=83301);  %sp(_n="104767",_p=63714);  %sp(_n="104171",_p=46950);  %sp(_n="103959",_p=75186);   
  %sp(_n="105052",_p=88906);  %sp(_n="100903",_p=11929);  %sp(_n="105319",_p=76415);  %sp(_n="101861",_p=87409); 
  %sp(_n="101382",_p=85866);  %sp(_n="102504",_p=89027);  
run;
  
proc sort data=cases_ccm1a; by NID _dist; run;
data cases_ccm1b; set cases_ccm1a; by NID; retain _diffidx;
  if first.NID then _diffidx=0;   _diffidx= _diffidx+1;
  if length(strip(comnam)) LE 3 then _diffidx=100;
  if _diffidx LE 1 then output; run;
  proc print data=cases_ccm1b(obs=30); var NID defendant comnam _dist ticker ticker_crsp tickerdiff; run;

proc sort data=cases_ccm1b out=cases_ccm1c; by _dist; run;
data cases_ccm2; set cases_ccm1c; if _dist LT 1.11; run;



data ccmxpf_linktable; set crsp.ccmxpf_linktable; 
  if linkenddt=.E then linkenddt = '31DEC2099'd; run;

proc sql; create table cases_ccm3a as 
  select distinct c.*, l.gvkey
  from cases_ccm2(drop= _diffidx _tic manual word ) c 
    left join ccmxpf_linktable l on c.permno=l.lpermno
  and year(DateFile_FIC) between year(linkdt) - 2 and year(linkenddt) +2
  where gvkey is not missing;

proc sort data=cases_ccm3a nodupkeys; by nid permno gvkey; run;
data cases_ccm3a; set cases_ccm3a; by nid permno gvkey; if last.nid; run;

proc sql; create table courtsref as 
  select distinct Court_ref, substr(court_ref,length(scan(court_ref,1,' '))+1,  
  ( length(court_ref)-length(scan(court_ref,1,' '))))  as courtstate  
  from cases_ccm3a order by courtstate, court_ref;

data court_cir; 
  infile "fci_data/Circuits_Districts.txt" firstobs=2;
  informat court_ref $27.; informat court_cir $10.;
  input court_ref $ 1-27 court_cir $ 28-38;
run;

proc sort data=court_cir;  by court_ref; run;
proc sort data=courtsref;  by court_ref; run;
data courtsref2; merge courtsref(in=_a ) court_cir; by court_ref; if _a; run;

proc sort data=courtsref2;  by court_ref; run;
proc sort data=cases_ccm3a; by court_ref nid; run;

data cases_ccm3; 
  merge cases_ccm3a(in=_c drop=complaintURL_fic complainturl_ref 
    complainturl_other marketstatus tickerdiff tickerdiffidx) courtsref2; by court_ref; 
  YearFile_FIC=year(datefile_fic);  if _c; run;

data fci.cases_ccm3; set cases_ccm3;run;








* Case specific control variables;
data cases_cirl0; set cases_ccm3;
  format DateReview mmddyy10.;  if length( _DateReview) GT 5 then DateReview=input(_DateReview,mmddyy10.);
  yearfic= year(datefile_fic);
  cps=min(cps_fic,cps_ref); cpe=max(cpe_fic,cpe_ref);   format cps cpe mmddyy10.;
  court=court_ref; if find(court,"-99") GT 0 then court=court_fic;
  case_financial=0; if find(sector,"financial","i") GT 0 then case_financial=1;
  if datefile_fic=. then datefile_fic=datefile_ref; 
  * Tellabs;
  if datefile_ref=. then datefile_ref=datefile_fic;
  AfterTellabs=0; if datefile_ref GT "21Jun2007"d then AfterTellabs=1;
  cases_sample=0;  if t4_nonus=0 and is10b5=1 then cases_sample=1;   run;

proc sort data=cases_cirl0 nodupkey dupout=d1; by nid; run;  

proc sql; create table cases_sich as select distinct c.nid, f.sich
  from cases_cirl0 c inner join 
    (select distinct gvkey, datadate, sich from comp.funda) f
  on c.gvkey=f.gvkey and f.datadate between c.cps and c.cpe;

proc sort data=cases_sich nodupkey; by nid; run;

data cases_cirl1; merge cases_cirl0 cases_sich; by nid; run;

*cases dsf;
proc sql; create table casesdsf0 as select distinct c.nid, c.cps, c.cpe, 
  d.permno, d.date, d.ret, d.shrout, d.vol, d.prc
  from crsp.dsf d inner join cases_cirl0 c on c.permno=d.permno and 
  d.date between c.cps and c.cpe
  order by d.date, d.permno;

proc sort data=crsp.dsi(where=(year(date) ge 1992)) out=dsi0; by date; run;

data casesdsf1; merge casesdsf0(in=_c) dsi0(keep=date vwretd ewretd); by date; 
  if ret=. then ret=0;   xret = ret - vwretd;      mve= prc * shrout;
  turnover0 = vol/1000/shrout;  if _c; run;

*cases turnover_cp mret_cp beta_cp; 
proc sql; create table cases_avgtu0 as select distinct nid, 
  mean(turnover0) as turnover_cp, 
  exp(sum(log(1+vwretd)))-1 as mktret_cp,
  count(distinct date) as x, 
  exp(sum(log(1+xret)))-1 as mret_cp,
  min(date) as d1 format mmddyy10., max(date) as d2 format mmddyy10., cps, cpe
  from casesdsf1 group by nid order by nid;

proc sql; create table cases_mve10d as select distinct c.nid, c.cpe, c.date, c.mve
  from casesdsf1 c where intck("day", c.cpe, c.date) GE -10
  group by c.nid having intck("day", c.cpe, c.date) = min(intck("day", c.cpe, c.date)  )
  order by c.nid;

proc sort data=casesdsf1; by nid; run;
proc reg data=casesdsf1 outest=cases_betae1 
  noprint; by nid; 
  model ret = ewretd /noint; quit;

data cases_beta(drop=ewretd); set cases_betae1(keep=nid ewretd);
  beta_cp=ewretd; run;

*cases msf: retskw_cp retstd_cp minret_cp;
proc sql; create table casesmsf0 as select distinct c.nid, c.cps, c.cpe, c.sich,
  d.permno, d.date, d.ret, d.shrout, d.vol, d.prc
  from crsp.msf d inner join cases_cirl1 c on c.permno=d.permno and 
  d.date between c.cps and c.cpe
  order by d.date, d.permno;

proc sort data=crsp.msi(where=(year(date) ge 1992)) out=msi0; by date; run;

data casesmsf1; merge casesmsf0(in=_c) msi0(keep=date vwretd); by date;
  if _c; run;

proc means data=casesmsf0 n skewness stddev noprint;   var ret;  class nid;   
  output out=casesmsf1means 
  skewness=retskw_cp stddev=retstd_cp min=minret_cp n=n_mth;run;

*iorcrsp_cp;
proc sql; create table ihold0 as
  select distinct rdate, ticker, cusip, sum(shares/shrout2/1000) as iorcrsp 
  from tfn.s34 group by rdate, cusip, ticker;

proc sql; create table ihold1o as select distinct i.*, c.permno
  from ihold0 i inner join crsp.msenames c on i.cusip=c.ncusip;

data ihold1; set ihold1o; cusip6=substr(cusip,1,6); 
  if iorcrsp LE 0.05 then iorcrsp=0;
  if iorcrsp GT 1 then iorcrsp=1;  if iorcrsp = .  then iorcrsp=0;  run;

proc sql; create table cases_io0 as select distinct c.nid,
  mean(i.iorcrsp) as iorcrsp_cp
  from cases_cirl0 c inner join ihold1 i on c.permno=i.permno and 
    i.rdate between c.cps and cpe
  group by c.nid;

*Industry mean return;
*For all firms of same sich, falling into the class-period period, regardless of being sued or not;
proc sql; create table cases_cpsich1 as select nid, sich, permno, 
  exp(sum(log(1+ret -vwretd))) -1 as cpret 
  from casesmsf1
  group by nid, sich, permno;

proc means data=cases_cpsich1 mean stddev noprint;   var cpret;  class sich;   
  output    out=cases_cpsich2
  mean=sichret_cp stddev=sichretstd_cp ;run;

proc sql; create table cases_cpsich3 as select c.nid, c.sich, 
  s.sichret_cp,
  s.sichretstd_cp
  from cases_cpsich1 c inner join cases_cpsich2 s on c.sich=s.sich
  order by nid;

*salesgr_cp;
proc sql; create view comp0v as
  select GVKEY, fyear, datadate, at, sale, SICH, csho, prcc_f, ceq, fic, 
    ni, epspx,   dltt, sstk, dltis
  from comp.funda
  where consol='C' and popsrc='D' and datafmt='STD'  and indfmt='INDL' and at ne .
    and fyear between 1994 and 2014
    and year(datadate) LE 2014;

%include "winsor.sas";
%winsor(dsetin=comp0v, dsetout=comp0vw, byvar=fyear, type=winsor, pctl=1 99,
  vars=at sale csho prcc_f ceq ni dltt sstk dltis epspx);

data comp0vw; set comp0vw;
  if sstk=. then sstk=0; if dltis=. then dltis=0;  run;

proc sql; create table sale_cps as select distinct c.nid, 
  s.datadate as datadate_cps, s.sale as sale_cps, s.sich
  from comp0vw s inner join  cases_cirl0 c on s.gvkey=c.gvkey and 
   intck("month",c.cps,s.datadate) between -12 and -1 order by nid;

proc sql; create table sale_cpe as select distinct c.nid, 
  s.datadate as datadate_cpe, s.sale as sale_cpe
  from comp0vw s inner join  cases_cirl0 c on s.gvkey=c.gvkey and 
   intck("month",c.cpe,s.datadate) between 0 and 11 order by nid;



proc sql; create table cases_salesgr as select distinct c.nid,
 ( sale_cpe/sale_cps -1 ) / intck("year",datadate_cps,datadate_cpe) as salesgr_cp,
 ( sale_cpe/sale_cps -1 ) as salesgr_cptot,
 intck("year",datadate_cps,datadate_cpe) as years_cp
 from sale_cps c inner join sale_cpe s on c.nid=s.nid;

*;
proc sort data=cases_mve10d; by nid; run;
proc sort data=cases_avgtu0; by nid; run;
proc sort data=cases_io0; by nid; run;
proc sort data=casesmsf1means; by nid; run;
proc sort data=cases_salesgr; by nid; run;
proc sort data=cases_beta; by nid; run;

data cases_dmg0; merge
  cases_mve10d(keep=nid mve) 
  cases_avgtu0(keep=nid turnover_cp x mret_cp mktret_cp) 
  cases_io0(keep=nid iorcrsp_cp  )
  casesmsf1means(keep=nid retskw_cp retstd_cp minret_cp)
  cases_salesgr(keep=nid salesgr_cp) 
  cases_beta
  cases_cpsich3(keep=nid sichret_cp sichretstd_cp );
  by nid;
  damage_cp =0; 
  if mret_cp LT 0 and mve GT 0 then
    damage_cp = (-1) * mve * mret_cp * (1 - (1-turnover_cp)**x); 
  if iorcrsp_cp=. then iorcrsp_cp=0 ;
run;  

proc sort data=cases_dmg0 nodupkeys; by nid; run;

data cases_cirl2; merge cases_cirl1(in=_c) cases_dmg0 ;
  by nid;
  if _c; run;
data fci.cases_cirl2;   set cases_cirl2; run; 











***Step 05:                     ******************************;
***Firm-year level control variables            **************;
***Compustat, CRSP, TFN                        ***************;

*Read Compustat...;
proc sql; create view comp0v as
  select GVKEY, fyear, datadate, at, sale, SICH, csho, prcc_f, ceq, fic, 
    ni, epspx,   dltt, sstk, dltis
  from comp.funda
  where consol='C' and popsrc='D' and datafmt='STD'  and indfmt='INDL' and at ne .
    and fyear between 1994 and 2014
    and year(datadate) LE 2014;

%include "winsor.sas";
%winsor(dsetin=comp0v, dsetout=comp0vw, byvar=fyear, type=winsor, pctl=1 99,
  vars=at sale csho prcc_f ceq ni dltt sstk dltis epspx);

data comp0vw; set comp0vw;
  if sstk=. then sstk=0; if dltis=. then dltis=0;  run;

proc sql; create table comp0 as
  select distinct c.gvkey, c.fyear, c.datadate, c.sich, c.fic, 
    substr(put(c.sich,z4.),1,1) as sic1,
    log(c.csho*c.prcc_f+1) as lgmv, 
    (c.sale - cl.sale) / cl.at as salesgr,  
    (p1.sale - c.sale) / c.at as salesgr_p1, 
    cl.csho as csholag, c.csho as csho,    
    c.dltt/c.at as lev,
    c.epspx/c.prcc_f as eps2pr,
    (c.epspx-p1.epspx)/p1.prcc_f as deps2pr,
    ( p1.sstk + p1.dltis) / c.at as Financing,
    c.ceq/c.csho/c.prcc_f as B2M,
    case when c.NI GE 0 then 0 else 1 end as LOSS    
  from comp0vw c 
    left join comp0vw cl  on c.gvkey = cl.gvkey  and c.fyear =  cl.fyear + 1
    left join comp0vw cll on c.gvkey = cll.gvkey and c.fyear = cll.fyear + 2 
    left join comp0vw p1  on c.gvkey = p1.gvkey  and c.fyear =  p1.fyear -1
  where year(c.datadate) between 1990 and 2014 
    and c.csho GT 0 and c.prcc_f GT 0; quit;

proc sort data=comp0 nodupkeys; by gvkey datadate; run;

data comp0; set comp0;
  if financing=. then financing=0; if lev=. then lev=0; 
  absdeps2pr = abs(deps2pr);
  negdeps2pr = (deps2pr < 0);
run;

*Read circuit court ideology;
data compa1_pctd; 
  set fci.compa1elr_usca (keep=gvkey datadate fps circuitno statecode fps_state 
    pctd pnld pswd peld gdpgr unemp );
  fyear=year(datadate); if month(datadate) LE 5 then fyear=year(datadate)-1;  run;

proc sort data=compa1_pctd nodupkeys; by gvkey datadate; run;

*Take beginning of year value by moving datadate;
data compa1_pctd_p1; 
  set compa1_pctd;
  format datadate yymmddn8.; 
  datadate = intnx("year",datadate,1,"sameday"); run;
proc sort data=compa1_pctd_p1 nodupkeys; by gvkey datadate; run;


*Read District Court Data;
data compa1_dcd; 
  set fci.compa1elr_usdc (keep=gvkey datadate dcd); 
  fyear=year(datadate); if month(datadate) LE 5 then fyear=year(datadate)-1; run;

proc sort data=compa1_dcd nodupkeys; by gvkey datadate; run;

*Take beginning of year value by moving datadate;
data compa1_dcd_p1; 
  set compa1_dcd;
  format datadate yymmddn8.; 
  datadate = intnx("year",datadate,1,"sameday"); run;
proc sort data=compa1_dcd_p1 nodupkeys; by gvkey datadate; run;


data comp1; 
  merge comp0(in=_c) compa1_pctd_p1(in=_p) compa1_dcd_p1; 
  by gvkey datadate;  
  sic2=strip(substr(put(sich,z4.),1,2));  if sic2="" then sic2="00";  format circuit $8.;
  sic2_financial=0; if substr(sic2,1,1)="6" then sic2_financial=1;
  if substr(circuitno,1,1)="C" then Circuit=put(input(substr(circuitno,2),8.),2.);
  else Circuit=put(12,2.);
  label circuitno   ="Circuit of current year";
  if _c and _p; run;

%include "ccm.sas";
%ccm(inset=comp1,outset=ccm1);

*Repair permno;
proc sort data=ccm1 out=ccm1a(keep=gvkey datadate permno where=(permno ne .)); 
  by gvkey datadate;  run;

proc sql; create table ccm2 as select distinct gvkey, permno as permno1 from ccm1
  where permno ne . order by gvkey; quit;

data comp2o; merge comp1(in=_c) ccm1a; by gvkey datadate; if _c; run;
data comp2; merge comp2o(in=_c) ccm2;  by gvkey; 
  if permno=. then permno=permno1;
if _c and permno ne .; run;

*Read institutional holding;

*gets a dataset fci.io_timeseries;
%include "io_ts.sas";

proc sql; create table comp2_iorcrsp as 
  select distinct c.gvkey, c.datadate, c.permno, i.ior as iorcrsp 
  from comp2 c left join fci.io_timeseries i on c.permno =i.permno and
    intck("day",c.datadate,i.rdate) between -92 and 0
  group by c.gvkey, c.datadate, c.permno having 
    intck("day",c.datadate,i.rdate)= max(intck("day",c.datadate,i.rdate))
  order by c.gvkey, c.datadate;

*CRSP msf;
proc sql; create table msfmsi0 as 
  select distinct m.permno, m.date, m.prc,m.ret,m.vol,
  m.shrout, i.ewretd, i.vwretd, (m.ret-i.vwretd) as xret
  from crsp.msf m inner join crsp.msi i on m.date=i.date;

proc sql; create table msf0 as 
  select distinct c.datadate,m.permno,m.date, m.ret, 
    m.vwretd, m.ewretd,
    m.vol/10 as vol, m.vol/10/m.shrout as turncrsp, m.prc*m.shrout as mkval
  from msfmsi0 m inner join comp2 c on m.permno = c.permno and 
    intck('month',c.datadate,m.date) between -11 and 0
  where ret is not missing;

%include "winsor.sas";
%winsor(dsetin=msf0, dsetout=msf0w, type=winsor, pctl=1 99,  vars=ret);

proc sql; create table msf1 as 
  select distinct datadate, permno, 
    exp(sum(log(1+vwretd))) -1 as mktret, 
    exp(sum(log(1+ret-vwretd))) -1 as ret, 
    exp(sum(log(1+ret))) - 1 as ret0, 
    sum(vol) as vol,
    sum(turncrsp) as turnover,
    max(date) as date_endcmpnd format yymmddn8. , count(*) as n_mth_cmpnd
  from msf0w group by permno, datadate;

data msf1_mkval(keep=permno datadate mkval); 
  set msf0(where=(intck("month",datadate,date)=0) ); run;

*Return skewness and volatility;
proc means data=msf0 n skewness stddev noprint;   var ret;  class permno datadate;   
  output out=msf1means skewness=retskw stddev=retstd n=n_mth;run;

data msf1; set msf1; if n_mth_cmpnd eq 12; run; 
data msf1means; set msf1means; if _FREQ_=12; run;    

*CRSP dsf;
proc sql; create table dsf0 as select distinct c.datadate, m.permno, m.date, m.ret
  from comp2 c inner join crsp.dsf m on m.permno=c.permno and
    intck('month',c.datadate,m.date) between -11 and 0
  where ret is not missing order by date, permno;

data dsi0; set crsp.dsi(where=(year(date) GE 1992)); run;
proc sort data=dsi0; by date; run;

data dsf0i; merge dsf0(in=_f) dsi0(in=_i keep=date vwretd ewretd); by date; if _f and _i; run;

proc sql; create table dsf1o as select distinct datadate, permno, min(ret) as minret,
  count(*) as daysminret
  from dsf0 group by datadate, permno
  order by permno, datadate;

data dsf1; set dsf1o; 
  if daysminret GE 120; run;
*NOTE: The data set WORK.DSF1 has 108 527 =>107 755 observations and 4 variables.;

proc sql; create table dec0 as select distinct c.datadate, m.permno, m.date, m.decret
  from comp2 c inner join crsp.erdport1 m on m.permno=c.permno and 
    intck('month',c.datadate,m.date) between -11 and 0;
*NOTE: The data set WORK.DEC0 has 26 983 287 observations and 4 variables.;

proc sort data=dsf0i; by permno datadate date; run;
proc sort data=dec0;  by permno datadate date; run;
 
data dec1; merge dsf0i(in=_d) dec0(in=_c); by permno datadate date; if _d; run;

proc reg data=dec1 outest=regbetae1 noprint; by permno datadate; 
  model ret = ewretd /noint; quit;

*Merge CRSP files into one;
proc sort data=msf1       nodupkeys; by permno datadate; run;
proc sort data=dsf1       nodupkeys; by permno datadate; run;
proc sort data=regbetae1  nodupkeys; by permno datadate; run;
proc sort data=msf1_mkval nodupkeys; by permno datadate; run;
proc sort data=msf1means  nodupkeys; by permno datadate; run;
proc sort data=comp2      nodupkeys; by permno datadate; run;
proc sort data=comp2_iorcrsp nodupkeys; by permno datadate; run;

data crsp0; merge msf1 dsf1 msf1means msf1_mkval 
  regbetae1(keep=permno datadate ewretd)
  comp2_iorcrsp(keep=permno datadate  iorcrsp); 
by permno datadate; run;

proc sort data=comp2      nodupkeys; by permno datadate; run;
data ccm1raw0(drop=ewretd); 
  merge comp2(in=_c) crsp0;    by permno datadate;
  betae=ewretd;
  nonus=0; if find(fic,"USA","i")=0 then nonus=1;
  if iorcrsp=. then iorcrsp=0;
  if year(datadate) GE 1990 and _c; run;

%include 'winsor.sas';
%winsor(dsetin=ccm1raw0, dsetout=ccm1wn, byvar=fyear, 
  vars= lgmv  salesgr ret retskw retstd turnover iorcrsp betae 
  lev eps2pr deps2pr financing absdeps2pr , 
type=winsor, pctl=1 99);

*Industry level variables;
%let indcode=sich;
proc sql; create table ccm1wn_&indcode. as select distinct datadate, &indcode., 
  mean(lgmv) as &indcode.lgmv,      mean(salesgr) as &indcode.salesgr,
  mean(ret) as &indcode.ret, mean(retskw) as &indcode.retskw,  
  mean(retstd) as &indcode.retstd,   mean(turnover) as &indcode.turnover
  from ccm1wn group by datadate, &indcode. order by datadate, &indcode.;

proc sql; create table ccm1wn_&indcode.vwret0 as select distinct datadate, &indcode., 
  sum(ret*mkval)/sum(mkval) as &indcode.vwret
  from ccm1wn group by datadate, &indcode. order by datadate, &indcode.;

proc sort data=ccm1wn; by datadate &indcode.; run;
proc sort data=ccm1wn_&indcode; by datadate &indcode.; run;


data ccm1w; merge ccm1wn ccm1wn_&indcode. ccm1wn_&indcode.vwret0; 
  by datadate &indcode.; run;

proc sort data=ccm1w nodupkey; by gvkey datadate; run;

data fci.ccm1w; set ccm1w; run;

***Step 06:                     ******************************;
***Merging firm-year with lawsuits, Aftertellabs *************;
***Dataset for main test, Table 5, 6, 7, 8        ************;

data ccm1w; set fci.ccm1w; run;

%winsor(dsetin=ccm1w, dsetout=ccm1w, type=winsor, pctl=1 99, vars=lgmv salesgr betae retskw retstd ret minret turnover 
   iorcrsp lev financing sichret sichretstd mktret gdpgr unemp peld);

*datadate is the fiscal year of control variables, year t-1;
*[cps, cpe] the class period overlaps with year t;
proc sql; create table casesccm_main_0 as
  select m.*, c.nid, c.aftertellabs, c.isipo, c.case_financial, c.datefile_fic, c.datefile_ref, c.court_cir
  from  ccm1w m 
    left join fci.cases_cirl2(where=(cases_sample eq 1)) c 
    on m.gvkey=c.gvkey and
   intck("day",c.cps,m.datadate) GE -365 and intck("day",c.cpe,m.datadate) LE -1;
quit;

proc sort data=casesccm_main_0 nodupkey; by gvkey datadate; run;

data casesccm_main; set casesccm_main_0;
  sued=0;  if nid ne '' then sued=1; 
  slusa=1;
  if datefile_fic ne . then do;  if year(datefile_fic) lt 1998 then slusa=0; end;
  if year(datadate) lt 1997 then slusa=0;   
  if n(fps, pnld, sued, lgmv, salesgr, ret, retskw, retstd, turnover) =9 and length(strip(sic2)) GE 1
   and n(betae, retskw, retstd, ret, minret, turnover, iorcrsp, sichret, sichretstd, mktret) = 10
   and year(datadate) LE 2013 ; 
  run;

data fci.casesccm_main; set casesccm_main; run;

*Stata output for main test dataset;
proc export data=casesccm_main file="&output_dir.casesccm_main.dta" dbms=stata replace; run; 

*Tellabs full sample match;
proc sql; create table casesccm_tellabs_0 as
  select m.*, c.nid, c.aftertellabs, c.isipo, c.case_financial, c.datefile_fic, c.datefile_ref
  from  fci.ccm1w m 
    left join fci.cases_cirl2(where=(cases_sample eq 1)) c 
    on m.gvkey=c.gvkey and
   intck("day",c.cps,m.datadate) GE -365 and intck("day",c.cpe,m.datadate) LE -1;
quit;

proc sort data=casesccm_tellabs_0 nodupkey; by gvkey datadate; run;

data casesccm_tellabs1; set casesccm_tellabs_0;
  sued=0;  if nid ne '' then sued=1; 
  slusa=1;
  if datefile_fic ne . then do;  if year(datefile_fic) lt 1998 then slusa=0; end;
  if year(datadate) lt 1997 then slusa=0;   
  if n(fps, pnld, sued, lgmv, salesgr, ret, retskw, retstd, turnover) =9 and length(strip(sic2)) GE 1
   and n(betae, retskw, retstd, ret, minret, turnover, iorcrsp, sichret, sichretstd, mktret) = 10
   and year(datadate) LE 2013 ; 
  run;

proc logistic data=casesccm_tellabs1 outest=estphat covout;
   model sued (event='1') = fps lgmv salesgr ret retskw retstd turnover betae minret iorcrsp lev financing;
   output out=casesccm_tellabs2 p=phat lower=lcl upper=ucl predprob=(individual crossvalidate);
run;  

data casesccm_sued;  
  set casesccm_tellabs2(where=(sued=1) keep=gvkey datadate fyear sic1 sic2 circuitno phat sued nid aftertellabs ); run;

data casesccm_mch0;  
  set casesccm_tellabs2(where=(sued=0) keep=gvkey datadate fyear sic1 sic2 circuitno phat sued lgmv);
  k+1;  run;

proc sort data=casesccm_sued nodupkeys; by nid datadate gvkey; run;

*Using proc sql left join to get full match;
proc sql; create table match1a as select m.gvkey, m.datadate, m.sic2, m.circuitno, m.phat, m.lgmv,
  m.fyear, m.sic1,
  s.aftertellabs, s.phat as phat_s, s.sic2 as sic2_s, s.datadate as datadate_s,
  abs(m.phat-s.phat) as absdiff_phat
  from casesccm_mch0 m left join casesccm_sued s
  on m.fyear = s.fyear and m.sic2=s.sic2
  group by m.gvkey, m.datadate having absdiff_phat = min(absdiff_phat)
  order by m.gvkey, m.datadate;

data match1bo; set   match1a; if aftertellabs=.; run;

proc sql; create table match1b as select m.gvkey, m.datadate, m.sic2, m.circuitno, m.phat, m.lgmv, m.fyear,
  s.aftertellabs, s.phat as phat_s, s.sic2 as sic2_s, s.datadate as datadate_s,
  abs(m.phat-s.phat) as absdiff_phat
  from match1bo m left join casesccm_sued s
  on m.fyear = s.fyear and m.sic1=s.sic1
  group by m.gvkey, m.datadate having absdiff_phat = min(absdiff_phat)
  order by m.gvkey, m.datadate;

data match1co; set   match1b; if aftertellabs=.; run;

proc sql; create table match1c as select m.gvkey, m.datadate, m.sic2, m.circuitno, m.phat, m.lgmv, m.fyear,
  s.aftertellabs, s.phat as phat_s, s.sic2 as sic2_s, s.datadate as datadate_s,
  abs(m.phat-s.phat) as absdiff_phat
  from match1co m left join casesccm_sued s
  on m.fyear = s.fyear 
  group by m.gvkey, m.datadate having absdiff_phat = min(absdiff_phat)
  order by m.gvkey, m.datadate;

data match1; 
  set match1a(where=(aftertellabs ne .)) match1b(where=(aftertellabs ne .)) match1c(where=(aftertellabs ne .)); 
  run;
proc sort data=match1 nodupkeys; by gvkey datadate ; run;
  
proc sort data=casesccm_mch0 nodupkeys; by gvkey datadate ; run;


data casesccm_mch1; merge casesccm_mch0(in=_c) match1(in=_m); by gvkey datadate;  run;

data tellabsFull0; set casesccm_sued casesccm_mch1; 
  pnld_after = pnld * aftertellabs;
run;

proc export data=tellabsFull0 file="&output_dir.tellabsFull.dta" 
  dbms=stata replace; run;



*Tellabs one to one match;
proc sql; create table casesccm_tellabs_0 as
  select m.*, c.nid, c.aftertellabs, c.isipo, c.case_financial, c.datefile_fic, c.datefile_ref
  from  fci.ccm1w m 
    left join fci.cases_cirl2(where=(cases_sample eq 1)) c 
    on m.gvkey=c.gvkey and
   intck("day",c.cps,m.datadate) GE -365 and intck("day",c.cpe,m.datadate) LE -1;
quit;

proc sort data=casesccm_tellabs_0 nodupkey; by gvkey datadate; run;

data casesccm_tellabs1; set casesccm_tellabs_0;
  sued=0;  if nid ne '' then sued=1; 
  slusa=1;
  if datefile_fic ne . then do;  if year(datefile_fic) lt 1998 then slusa=0; end;
  if year(datadate) lt 1997 then slusa=0;   
  if n(fps, pnld, sued, lgmv, salesgr, ret, retskw, retstd, turnover) =9 and length(strip(sic2)) GE 1
   and n(betae, retskw, retstd, ret, minret, turnover, iorcrsp, sichret, sichretstd, mktret) = 10
   and year(datadate) LE 2013 ; 
  run;

proc logistic data=casesccm_tellabs1 outest=estphat covout;
   model sued (event='1') = fps lgmv salesgr ret retskw retstd turnover betae minret iorcrsp lev financing;
   output out=casesccm_tellabs2 p=phat lower=lcl upper=ucl predprob=(individual crossvalidate);
run;  

data casesccm_sued;  
  set casesccm_tellabs2(where=(sued=1) keep=gvkey datadate fyear sic1 circuitno phat sued nid aftertellabs ); run;

data casesccm_mch0;  
  set casesccm_tellabs2(where=(sued=0) keep=gvkey datadate fyear sic1 circuitno phat sued
                     rename=(phat=_phat gvkey=_gvkey));
  k+1;  run;

proc sort data=casesccm_sued; by  phat; run;
proc sort data=casesccm_mch0; by _phat; run;

*Below the _gvkey datadate is the matched non-sued obs;
data casesccm_mch1; 
  if _n_ eq 1 then do;
    if 0 then set casesccm_mch0;
    declare hash h(dataset:'casesccm_mch0',multidata:'y');
    h.definekey('fyear','sic1');
    h.definedata('_gvkey','_phat','k');
    h.definedone();   end; 
  call missing(of _all_);
  set casesccm_sued;
  n=0;   min=9999999;    rc=h.find();
  do while(rc=0);
    abs=abs(phat-_phat);
    if abs lt min then do;n=k;min=abs;end;
    rc=h.find_next();    end;
  rc=h.find();
  do while(rc=0);
    if n=k then do;h.removedup();leave;      end;
    rc=h.find_next();    end;
  drop rc n min abs k;   _match=1;run;

proc sort data=casesccm_mch1 nodupkeys; by _gvkey fyear; run;
proc sort data=casesccm_mch0 nodupkeys; by _gvkey fyear; run;

data casesccm_mch2; 
  merge casesccm_mch0 casesccm_mch1(keep=_gvkey fyear nid aftertellabs in=_m); by _gvkey fyear;
  if _m then do; 
    sued=0; gvkey=_gvkey; end; if _m; run;

data casesccm_match0; 
  set casesccm_mch2(keep=gvkey datadate aftertellabs sued nid in=_m)  
      casesccm_sued(keep=gvkey datadate aftertellabs sued nid); 
run;

proc sort data=casesccm_match0 out=casesccm_match1 nodupkeys dupout=_d; by gvkey datadate sued; run;
proc sort data=casesccm_main                       nodupkeys; by gvkey datadate; run;

data casesccm_match2; 
  merge casesccm_tellabs2(drop=aftertellabs) casesccm_match1(keep=gvkey datadate aftertellabs in=_a); by gvkey datadate; 
  pnld_after=pnld*aftertellabs;
  if _a; run;

proc reg data=casesccm_match2;
  model sued = pnld pnld_after aftertellabs 
    fps lgmv salesgr betae retstd retskw ret minret turnover iorcrsp lev financing 
    sichret sichretstd mktret gdpgr unemp peld; run;
data fci.casesccm_tellabs1to1; set casesccm_match2; run;

*Stata output for tellabs one to one match sample; 
proc export data=casesccm_match2 file="&output_dir.casesccm_tellabs1to1.dta" dbms=stata replace; run; 





***Step 07:                     ******************************;
***Table 10, 11 IBES CIG Data, 8-k filings       *************;
***Calculating voluntary and mandatory disclosure variable****;

data dhgdet0; 
  infile "fci_data/DHGDET.US" delimiter="|" MISSOVER DSD lrecl=32767 firstobs=2;
    format ibes_tkr periodicity measure currency range_desc diff_code act_std guidance_code action $8.;
    format units _activation_dt _announce_dt _mod_dt $30.;
    format prd_yr prd_mon eefymo 4.; 
    format val_1 val_2 mean_at_date 27.10;
    format activation_dt announce_dt datadateq_ibes datadate_ibes  mmddyy10.;
  input ibes_tkr $ prd_yr  prd_mon eefymo periodicity $ 
          measure $ val_1 val_2 currency $ units $ 
        range_desc $ _activation_dt $ _announce_dt $ _mod_dt $ diff_code $ 
         act_std $ mean_at_date action guidance_code;
  activation_dt=input(substr(_activation_dt,1,10),mmddyy10.);
  announce_dt  =input(substr(_announce_dt  ,1,10),mmddyy10.);
  prd_day=31;  if prd_mon=2 then prd_day=28;
  if prd_mon=4 or prd_mon=6 or prd_mon=9 or prd_mon=11 then prd_day=30;
  datadateq_ibes = mdy(prd_mon, prd_day, prd_yr); 
  myyr=prd_yr; if prd_mon GT eefymo then myyr=myyr+1;
  prd_day=31;  if eefymo=2 then prd_day=28;
  if eefymo=4 or eefymo=6 or eefymo=9 or eefymo=11 then prd_day=30;
  datadate_ibes = mdy(eefymo, prd_day, myyr); 
  drop prd_day myyr; 
  hy=prd_yr - year(announce_dt);
  hm= prd_mon - month(announce_dt);
  horizon=(prd_yr - year(announce_dt))*365 + (prd_mon - month(announce_dt))*30 + 30 - day(announce_dt);
run;

data dhgdet1; 
  set dhgdet0(where=(measure="EPS" and units="P/S" 
    and (periodicity="ANN" or periodicity="QTR")) rename=(ibes_tkr=ticker)); run;

data casesccm_main; set fci.casesccm_main;  run;

proc logistic data=casesccm_main outest=_est covout;
   model sued (event='1') = fps lgmv salesgr ret retskw retstd turnover betae minret iorcrsp lev financing;
   output out=casesccm1 p=phat lower=lcl upper=ucl predprob=(individual crossvalidate);
run;  

proc sort data=casesccm1; by gvkey datadate; run;




*RDQ;
proc sql; create table compq_rdq0 as select distinct gvkey, datadate, fqtr, rdq, niq
  from comp.fundq
  where CONSOL='C' and INDFMT='INDL' and DATAFMT='STD' and POPSRC='D' and FIC='USA' 
  and fqtr=4 and fyearq GE 1992 and  intck("day",datadate,rdq) between 1 and 180;

proc sql; create table compq_rdq1 as select distinct c.gvkey, c.datadate, c.rdq, l.rdq as rdqlag,
  case when c.niq GE 0 then 0 else 1 end as LOSS
  from compq_rdq0 c inner join compq_rdq0 l on c.gvkey=l.gvkey and
  intck("month",l.datadate,c.datadate)=12
  order by c.gvkey, c.datadate; 

data casesccm2; merge casesccm1(in=_c) compq_rdq1(in=_r) ; 
  by gvkey datadate; 
  if _c and _r; run;

*IBES TICKER;
%include 'iccmlink.sas';
%ICCMLINK;

proc sql; create table _ticker as select distinct c.permno, c.datadate, l. ticker
  from casesccm1 c inner join iccmlink l on c.permno=l.permno 
  and c.datadate between l.linkdt and l.linkenddt
  order by c.permno, c.datadate;

proc sort data=casesccm2; by permno datadate; run;
data casesccm3; merge casesccm2(in=_c) _ticker(in=_l); by permno datadate; if _c and _l; run;

*IBES Annual news;
proc sql; create table statsumu0 as select distinct c.permno, c.datadate, c.rdq, c.rdqlag,
  i.ticker, i.fpedats, i.statpers, i.meanest, i.medest, i.stdev
  from ibes.statsumu_epsus i inner join casesccm3 c on i.ticker=c.ticker and
    intck("month", c.rdqlag, i.statpers) = 1
  where i.fiscalp="ANN" and i.FPI="1" and i.estflag="P"
  order by c.permno, c.datadate; 
    *intck("month",i.statpers, c.datadate) = 7;

proc sql; create table statsumu1 as select c.*, a.value as act
  from statsumu0 c left join ibes.actu_epsus a
      on c.ticker=a.ticker and c.fpedats=a.pends and a.pdicity='ANN'
  order by c.permno, c.datadate; 

proc sql; create table statsumu2 as select c.*, m.prc, m.date as prcdate
  from statsumu1 c left join crsp.msf m on c.permno =m.permno and 
    intck("month",c.statpers, m.date) = 0
  order by c.permno, c.datadate; 

data statsumu3; set statsumu2;
  AnnNewsN= (act - meanest)/prc;    AnnNewsD= (act - medest )/prc;   
  if annnewsn>1 then annnewsn=1;    if annnewsn<-1 then annnewsn=-1;
  if prc GE 1; run;

*CIG Annual Long-term guidance;
proc sql; create table ibescigann0 as select g.ticker, g.announce_dt, g.datadate_ibes, 
  g.val_1, g.val_2, g.range_desc, g.guidance_code
  from dhgdet1 g where g.measure="EPS" and g.units="P/S" and g.periodicity="ANN";

proc sql; create table ibescigann1 as select g.*, i.fpedats, i.meanest, i.medest
  from ibescigann0 g inner join  ibes.statsumu_epsus i on i.ticker=g.ticker and 
  intck("month",i.statpers,g.announce_dt) =1 and
  intck("month",g.datadate_ibes,i.fpedats)=0
  where  i.fiscalp="ANN" and i.estflag="P";             *The prior month consensus; 

data ibescigann1; set ibescigann1; 
  cigvl=val_1; if val_2 NE . then cigvl=(val_1+val_2)/2;
  horizonann= intck("day",announce_dt, datadate_ibes);
  if horizonann GE 0 then lghorizonann= log(1+intck("day",announce_dt, datadate_ibes));
  cigupann=( cigvl > meanest); 
run;

proc sort data=ibescigann1 nodupkeys; by ticker datadate_ibes announce_dt; run;

*Annual long-term guidance: beyond current year;
proc sql; create table ibescigann1_sum as select s.permno, s.datadate, s.ticker, 
  sum(cigupann) as numgupanl
  from statsumu3 s inner join ibescigann1 g on s.ticker=g.ticker 
    and intck("day", s.rdq,    g.announce_dt) LT 0
    and intck("day", s.rdqlag, g.announce_dt) GE 1
    and intck("day",announce_dt, datadate_ibes) GT 90
  group by s.permno, s.datadate order by s.permno, s.datadate;

%include "winsor.sas";
%winsor(dsetin=ibescigann1_sum,   dsetout=ibescigann1_sumw, type=winsor, pctl=1 99,  vars=numgupanl);

proc sort data=ibescigann1_sumw nodupkeys; by permno datadate; run;

data ibescigann1_lag; 
  set ibescigann1_sumw(keep=permno datadate  numgupanl rename=( numgupanl=numlaggupanl));
  datadate=intnx("year",datadate,1,"same"); run;



*CIG QTR Short-term guidance;
proc sql; create table ibescigqtr0 as select g.ticker, g.announce_dt, g.datadate_ibes, 
  g.val_1, g.val_2, g.range_desc, g.guidance_code
  from dhgdet1 g where g.measure="EPS" and g.units="P/S" and g.periodicity="QTR";

proc sql; create table ibescigqtr1 as select g.*, i.fpedats, i.meanest, i.medest
  from ibescigqtr0 g inner join  ibes.statsumu_epsus i on i.ticker=g.ticker and 
  intck("month",i.statpers,g.announce_dt) =1 and
  intck("month",g.datadate_ibes,i.fpedats)=0
  where  i.fiscalp="QTR" and i.estflag="P";             *The prior month consensus; 

data ibescigqtr1; set ibescigqtr1; 
  cigvl=val_1; if val_2 NE . then cigvl=(val_1+val_2)/2;
  horizonqtr= intck("day",announce_dt, datadate_ibes);
  hormthqtr= intck("month",announce_dt, datadate_ibes);
  if horizonqtr GE 0 then lghorizonqtr= log(1+intck("day",announce_dt, datadate_ibes));
run;

proc sort data=ibescigqtr1 nodupkeys; by ticker datadate_ibes announce_dt; run;

proc sql; create table ibescigqtr1_sum as select s.permno, s.datadate, s.ticker, 
     count(*) as numcigqts
  from statsumu3 s inner join ibescigqtr1 g on s.ticker=g.ticker 
    and intck("day", s.datadate,    g.announce_dt) LT 0
    and intck("day", s.rdqlag,      g.announce_dt) GE 1
    and  intck("day",announce_dt, datadate_ibes) LE 90 
  group by s.permno, s.datadate order by s.permno, s.datadate;

%include "winsor.sas";
%winsor(dsetin=ibescigqtr1_sum,   dsetout=ibescigqtr1_sumw, type=winsor,  pctl=1 99,  vars=numcigqts);

proc sort data=ibescigqtr1_sumw nodupkeys; by permno datadate; run;

data ibescigqtr1_lag; 
  set ibescigqtr1_sumw(keep=permno datadate  numcigqts rename=(numcigqts=numlagcigqts));
  datadate=intnx("year",datadate,1,"same");
  run;

*Merge;
proc sort data=ibescigqtr1_sumw nodupkeys; by permno datadate; run;
proc sort data=ibescigqtr1_lag  nodupkeys; by permno datadate; run;
proc sort data=ibescigann1_sumw  nodupkeys; by permno datadate; run;
proc sort data=ibescigann1_lag   nodupkeys; by permno datadate; run;
proc sort data=statsumu3         nodupkeys; by permno datadate; run;
proc sort data=casesccm3         nodupkeys; by permno datadate; run;

%macro repair_log(v=);
  if num&v=. then num&v=0; log&v=log(1+num&v); %mend;

data ibescig2merge; 
  merge casesccm3(in=_c)  statsumu3(in=_i)  ibescigqtr1_sumw  ibescigann1_sumw 
    ibescigann1_lag
    ibescigqtr1_lag    ;
  by permno datadate;
  %repair_log(v=cigqts); 
  %repair_log(v=lagcigqts);
  %repair_log(v=gupanl);
  %repair_log(v=laggupanl);
  if _c and _i ; run;

proc sort data=ibescig2merge nodupkeys; by permno datadate; run;
data fci.ibescig2; set ibescig2merge; run;

*Read 8K item files;

%macro setitm(t=,no=);  
  if find(itemname,&t,"i") GT 0 then item&no.=1;  %mend;

%macro ReadEikItemByfy(filingyear=);
  %do filingyear =1996 %to 2015;
  data ei&filingyear. (keep=filingyear cik eikdate itemm ); 
    infile "fci_data/eikitem&filingyear..txt"  delimiter='09'x MISSOVER DSD lrecl=32767 firstobs=2;
    informat cik $10.; informat eikdate yymmdd10.; informat itemname $2048.;  informat _tdate $12.; 
    format eikdate yymmddn8.;
    informat isFwlook numsent v1 v2 v3 v4 v5 v6 v7 10.;  
    input cik $ _tdate $ itemname $ isFwlook numsent v1 v2 v3 v4 v5 v6 v7  ;
    eikdate=input(_tdate,yymmdd8.);               
    itemname=strip(itemname);    if length(itemname) LT 2 then itemname="null";   
    item101=0; item102=0; item103=0; item104=0; item201=0; item202=0; item203=0; item204=0; item205=0; item206=0;
    item301=0; item302=0; item303=0; item401=0; item402=0; item501=0; item502=0; item503=0; item504=0; item505=0; 
    item506=0; item507=0; item508=0; item701=0; regfd=0;item801=0; item901=0;    
    if find(itemname,"entry into","i") GT 0 and find(itemname,"agreement","i") GT 0 then item101=1;
    if find(itemname,"termination","i") GT 0 and find(itemname,"agreement","i") GT 0 then item102=1;
    %setitm(t="bankrupt",no=103);                           %setitm(t="mine safety",no=104);
    %setitm(t="Completion of Acquisition",no=201);          %setitm(t="Results of Operations",no=202);
    %setitm(t="Creation of a Direct",no=203);               %setitm(t="triggering events that",no=204);
    %setitm(t="Cost associated with exit",no=205);          %setitm(t="material impairment",no=206);
    %setitm(t="Notice of delisting or",no=301);             %setitm(t="unregistered sales of",no=302);
    %setitm(t="material modification to rights",no=303);    %setitm(t="Certifying Accountant",no=401);
    if find(itemname,"Non-reliance on","i") GT 0      or    find(itemname,"previously issued","i") GT 0 then item402=1;
    %setitm(t="Changes in control",no=501);                 %setitm(t="departure of directors",no=502);
    %setitm(t="amendments to articles",no=503);             %setitm(t="temporary suspension of trading",no=504);
    %setitm(t="amendment to registrant's code of",no=505);  %setitm(t="change in shell company status",no=506);
    %setitm(t="submission of matters to a vote",no=507);    %setitm(t="shareholder director nominations",no=508);
    if find(itemname,"Regulation FD Disclosure","i") GT 0 then item701=1;
    if find(itemname,"other event","i") GT 0     then item801=1;
    if find(itemname,"Financial Statements and E","i") GT 0 then item901 =1;
  filingyear = year(eikdate);
  items=item101+item102+item103+item104+item201+item202+item203+item204+item205+item206+item301+item302+item303
    +item401+item402+item501+item502+item503+item504+item505+item506+item507+item508+item701+item801+item901;
  itemm=item101+item102+item103+item104+item201        +item203+item204+item205+item206+item301+item302+item303
    +item401+item402+item501+item502+item503+item504+item505+item506+item507+item508                +item901;
  run;
  %end;
  data eikitem;
    set
      %do filingyear =1996 %to 2015; ei&filingyear   %end;
    ;
  run;
%mend ReadEikItemByfy;

%readeikitembyfy; 

proc sort data=eikitem out=eikitem; by cik eikdate;run;

data eikbyfile;
  set eikitem; by cik eikdate;
  retain maneik;
  if first.eikdate then do;   maneik=0;   end;
  if itemm GE 1  then maneik=1;
  if last.eikdate then output;  run;

data didx0(drop=_eikdate); 
  infile "fci_data/didx_form8K_19942016.csv" delimiter="," MISSOVER DSD lrecl=32767 firstobs=1;
  format cName $128.;     format fileType $16.;       format CIK $10.;     
  format _eikdate $8.;    format eikdate yymmddn8.;
  format fileURL $128.;  
    input cName $ fileType $ CIK $ _eikdate $ fileURL $ ;
  eikdate=input(_eikdate, yymmdd8.);
  if find(filetype,"8-K","i") GT 0; run;

proc sort data=didx0 nodupkey; by cik eikdate; run;
proc sort data=eikbyfile;      by cik eikdate; run;

data eightk0; 
  merge didx0 eikbyfile(in=_e); by cik eikdate; 
  format eikdate yymmddn8.;
  if _e; run;

*Compustat cik-gvkey;
proc sql; create table comp0cik as 
  select gvkey, fyear, datadate, CIK, cusip
  from comp.funda where consol='C' and popsrc='D' and datafmt='STD' and indfmt='INDL';

*Get RDQ from Comp Fundq;
data compq0; set comp.fundq(keep=gvkey fyearq datadate consol popsrc datafmt indfmt rdq);
  where consol="C" and popsrc="D" and datafmt="STD" and indfmt="INDL" ;
  keep gvkey fyearq datadate rdq;  
  if intck("day",datadate,rdq) ge 0 and intck("day",datadate,rdq) le 180;  run;

proc sort data=comp0cik; by gvkey datadate; run;
proc sort data=compq0;   by gvkey datadate; run;
data comp0cik1(drop=fyearq); 
  merge comp0cik(in=_a) compq0; by gvkey datadate; if _a; run;

*Get firm-year file with [rdqlag, rdq];
proc sql; create table comp0cik2 as
  select c.gvkey, c.fyear, c.datadate, c.cusip,c.cik, c.rdq, l.rdq as rdqlag, l.datadate as datadatelag
  from comp0cik1 c inner join 
       comp0cik1 l on c.gvkey=l.gvkey and c.fyear=l.fyear+1
       and intck("day",l.datadate, c.datadate) between 360 and 370
  where c.rdq is not missing and l.rdq is not missing and c.cik is not missing;

data comp0cik3; set comp0cik2;
  cik0=cik;
  cik1=substr(cik,1,1);
  do while (cik1="0");
    cik=substr(cik,2);
    cik1=substr(cik,1,1);
  end; 
run;

*Merge compustat with 8k;
proc sql; create table comp0eightk0 as 
  select distinct c.*, e.*
  from comp0cik3(drop=cik1 cik0) c inner join eightk0  e 
  on c.cik=e.cik and e.eikdate between c.rdqlag -1 and c.rdq -2
  order by gvkey, datadate,eikdate;

proc sort data=comp0eightk0 nodupkeys out=comp0eightk0s ; 
  by gvkey datadate eikdate; run;

%include "ccm.sas";
%ccm(inset=comp0eightk0,outset=eik0);

data eik0; set eik0(keep=gvkey datadate eikdate permno); run;

%include "evtstudy08.sas";
%EVTSTUDY(inset=eik0, evtdate=eikdate, id=permno, estper=110,model=madj,
  start=-2,end=2,gap=21,outset=eikcar0);

proc sort data=comp0eightk0 nodupkeys; by gvkey datadate eikdate; run;
proc sort data=eikcar0 nodupkeys;      by gvkey datadate eikdate; run;

data comp0eikcar; 
  merge comp0eightk0(keep=gvkey datadate eikdate rdqlag rdq maneik in=_e) 
        eikcar0(keep=gvkey datadate permno eikdate car); 
  by gvkey datadate eikdate; 
  nekpcar=(car>0);   nekncar=(car<=0); 
  if _e; run;

proc sql; create table comp0eik_annm0 as select distinct gvkey, datadate, permno,
 count(*) as numnekmnd, sum(nekpcar)     as numnekmndpcr,  sum(nekncar)     as numnekmndncr
 from comp0eikcar
 where maneik=1 
 group by gvkey, datadate
 order by gvkey, datadate;

%include "winsor.sas";
%winsor(dsetin=comp0eik_annm0,   dsetout=comp0eik_annm1, type=winsor, pctl=1 99, 
   vars=numnekmndpcr numnekmndncr numnekmnd);

data fci.comp0eik_annm1; set comp0eik_annm1(keep=gvkey permno datadate numnekmnd numnekmndpcr numnekmndncr);   run;

proc sort data=comp0eik_annm1 nodupkeys; by gvkey datadate; run;
data comp0eik_annm1_lag; 
  set comp0eik_annm1(keep=gvkey datadate numnekmnd numnekmndpcr numnekmndncr 
    rename=( numnekmnd=numlagnekmnd numnekmndpcr=numlagnekmndpcr numnekmndncr=numlagnekmndncr ));
  datadate=intnx("year",datadate,1,"same"); run;

*Firm Years;
data casesccm_main; set fci.casesccm_main;
  format ddtbk7m yymmddn8.;
  ddtbk7m=intnx("month",datadate,-7,"same"); 
  run;

proc logistic data=casesccm_main outest=_est covout;
   model sued (event='1') = fps lgmv salesgr ret retskw retstd turnover betae minret iorcrsp lev financing;
   output out=casesccm1 p=phat lower=lcl upper=ucl predprob=(individual crossvalidate);
run;  

proc sort data=casesccm1; by gvkey datadate; run;

*RDQ;
proc sql; create table compq_rdq0 as select distinct gvkey, datadate, fqtr, rdq, niq
  from comp.fundq
  where CONSOL='C' and INDFMT='INDL' and DATAFMT='STD' and POPSRC='D' and FIC='USA' 
  and fqtr=4 and fyearq GE 1992 and  intck("day",datadate,rdq) between 1 and 180;
*Table WORK.COMPQ_RDQ0 created, with 149444 rows and 4 columns.;

proc sql; create table compq_rdq1 as select distinct c.gvkey, c.datadate, c.rdq, l.rdq as rdqlag,
  case when c.niq GE 0 then 0 else 1 end as LOSS
  from compq_rdq0 c inner join compq_rdq0 l on c.gvkey=l.gvkey and
  intck("month",l.datadate,c.datadate)=12
  order by c.gvkey, c.datadate; 

*Merge Firm-year-8k;
%macro repair_log(v=);
  if num&v=. then num&v=0; log&v=log(1+num&v); %mend;

data casesccm_eik1; merge casesccm1(in=_c) compq_rdq1(in=_r) comp0eik_annm1 comp0eik_annm1_lag; 
  by gvkey datadate; 
  %repair_log(v=nekmnd);
  %repair_log(v=nekmndpcr);
  %repair_log(v=nekmndncr);
  %repair_log(v=lagnekmnd);
  %repair_log(v=lagnekmndpcr);
  %repair_log(v=lagnekmndncr);
  if _c and _r; run;

data fci.casesccm_eik1; set casesccm_eik1; run;

*Merge IBES disclosure with 8k disclosure;
data ibescig2; set fci.ibescig2; run;

data nekmnd; set  fci.casesccm_eik1; run;

proc sort data=ibescig2 nodupkeys; by gvkey datadate; run;
proc sort data=nekmnd   nodupkeys; by gvkey datadate; run;

data ibesnek0; merge ibescig2(in=_i) 
  nekmnd(keep=gvkey datadate lognekmnd loglagnekmnd lognekmndpcr lognekmndncr loglagnekmndpcr loglagnekmndncr in=_e); 
  by gvkey datadate;
  if _i and _e and 
    n(pnld, logcigqts, loglagcigqts, loggupanl, loglaggupanl, lognekmnd, loglagnekmnd, deps2pr, annnewsn) =9; run;

data fci.ibesnek0; set ibesnek0; run;
proc export data=ibesnek0     file="&output_dir.ibesnek.dta"  dbms=STATA replace; run;


***Step 08:                     ******************************;
***Table 9B, judge appointment and lawsuit filing     ********;
***Table 9C, market reaction to judge appointment test ********;

data fjc0; set fci.fjc_hist0;
  if substr(courttype,1,4)="USCA" and find(CircuitNo,"CF","i")=0
    and year(jdate1) GE 1996 and year(jdate1) LE 2015 
    and  intck("day","31Mar2015"d, jdate1) LE 0; run;

proc sort data=fjc0  out=fjc1 nodupkeys; by fjcjid; run;
proc sort data=fjc1  out=fjc1s ; by jdate1; run;

data casesccm_main1; 
  set fci.casesccm_main (keep=gvkey datadate fyear permno circuitno fps pnld  sued nid datefile_fic 
  lgmv salesgr ret retskw retstd  turnover iorcrsp mktret sichret sichretstd gdpgr unemp peld minret betae );
 run;

*-2, +2 years relative to judge appointment ;
proc sql; create table jappsue0 as select j.fjcjid, j.party, j.jdate1, j.courtname, c.*
  from fjc1s j left join casesccm_main1 c on strip(j.circuitno)=strip(c.circuitno) and 
  intck("year",j.jdate1,c.datadate) between -2 and 1;

proc sort data=jappsue0 nodupkey; by jdate1 gvkey datadate; run; 

data jappsue1; set jappsue0; 
  relYear = intck("year",jdate1,datadate,"C"); 
  afterApp=0; if  intck("day",jdate1,datadate) GE 1 then afterApp=1;  
  DemJudge=0; if party="D" then DemJudge=1;
  Dem_after = DemJudge*afterApp;
  run;


*** Market reaction to judge appointment;

*Judge appointment of USCA circuit courts during FY 1996 and 2014;
data fjc0; set fci.fjc_hist0;
  if substr(courttype,1,4)="USCA" and find(CircuitNo,"CF","i")=0
    and year(jdate1) GE 1996 and year(jdate1) LE 2015 
    and  intck("day","31Mar2015"d, jdate1) LE 0; run;

proc sort data=fjc0  out=fjc1 nodupkeys; by jdate1 fjcjid; run;

*appointment firm year match;
proc sql; create table compa1fjc0 as select 
  c.gvkey, c.datadate, c.permno, c.fyear, c.fps, c.pnld, c.unemp, c.gdpgr, c.peld, c.lgmv, c.betae, c.b2m, 
  c.circuitno, c.statecode,
  f.fjcjid, f.party,  f.jdate1, f.jdate2, f.retiredate
  from fci.ccm1w c inner join fjc0 f on c.circuitno=f.circuitno and
  intck("month",c.datadate,f.jdate1) between 1 and 12;

proc sort data=compa1fjc0 nodupkeys; by permno gvkey fjcjid; run;

*Additional controls;
proc sql; create table msfmsi0 as
  select distinct m.permno, m.date, m.prc,m.ret,m.vol,  m.shrout, i.ewretd, i.vwretd
  from crsp.msf m inner join crsp.msi i on m.date=i.date;

%include "winsor.sas";
%winsor(dsetin=msfmsi0, dsetout=msf0w, type=winsor, pctl=1 99,  vars=ret);

proc sql; create table msf1 as
  select distinct m.permno,  c.fjcjid, c.gvkey, c.datadate,
    exp(sum(log(1+m.ret-m.vwretd))) -1 as j6mret,
    max(date) as date_endj6 format yymmddn8. , count(*) as n_mth_cmpnd
  from compa1fjc0 c left join msfmsi0 m
    on c.permno=m.permno and intck("month", c.jdate1, m.date) between -6 and -1
  group by m.permno,  fjcjid, gvkey, datadate;

*Appointment CAR;
%include "evtstudy08.sas";
%EVTSTUDY(inset=compa1fjc0, evtdate=jdate1, id=permno, estper=110,model=madj,
  start=-1,end=1,gap=21,outset=carjapp_madj);

data carjapp; set carjapp_madj0(keep=gvkey permno datadate fjcjid party jdate1 car
   rename=(car=carmadj)); run;;
proc sort data=carjapp nodupkeys; by gvkey permno datadate fjcjid; run;

%include "winsor.sas";
%winsor(dsetin=carjapp, dsetout=carjappw,type=winsor, pctl=1 99,  vars=carmadj);

proc sort data=compa1fjc0 nodupkeys; by gvkey datadate fjcjid; run;
proc sort data=carjappw   nodupkeys; by gvkey datadate fjcjid; run;
proc sort data=msf1       nodupkeys; by gvkey datadate fjcjid; run;

data carjappw1; 
  merge carjappw(in=_j) msf1(keep=gvkey datadate fjcjid j6mret) compa1fjc0(in=_c);
  by gvkey datadate fjcjid; 
  pctcarmadj=carmadj*100;
  pctj6mret =j6mret*100;
  if _j and _c;
run;

proc export data=carjappw1 file="&output_dir.carjappw1.dta" dbms=stata replace; run; 
data fci.carjappw1; set carjappw1; run;



*** END ******************************;


